/****************************************************************************
 *
 * File Name: Dispatcher.h
 *
 * General Description: General description of the file.
 *
 * Created By: Alex Creation Date: 01/01/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/
#ifndef DISPATCHER_H
#define DISPATCHER_H
/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file Dispatcher.h
 *\brief A brief description of the file.
 *
 * Full description of the file.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "../Includes/concurrent_queue.h"
#include "../Includes/Datastructures.h"

class LLRE;
class NetworkDriver;
class GFCP_WS_Client;

/**
 * \brief Dispatcher class.
 */
class Dispatcher
{

public:
  Dispatcher();
  Dispatcher(NetworkDriver *_ndriver);
  virtual ~Dispatcher();
  virtual std::string sendFrame(Frame * frm);
  NetworkDriver *getNdriver() const;
  void setNdriver(NetworkDriver *ndriver);
  LLRE *getLlre() const;
  void setLlre(LLRE *llre);
  void setGfcp_client(GFCP_WS_Client *gfcpClient);
  void pushFrameToQueue(Frame *frame);

private:
  void startQueueProcessingThread();
  static void *processQueue(void *_disp);

  pthread_t threadPtr; //queue processing thread id
  NetworkDriver* ndriver; //network driver pointer
  LLRE* llre; //LLRE pointer
  GFCP_WS_Client* gfcpClient; //GFCP_WS_Client pointer
  concurrent_queue<Frame *> dispatcherQ; //thread safe frame queue
};

#endif /* DISPATCHER_H */
