/** \file GFCP_WS_Client.h
 \brief Defines the GFCP Web Services Client class.

 The GFCP_WS_Client is a module used for running with the GFCP web server.
 This module supplies with the available functions to be used


 */

#ifndef GFE_WS_CLIENT_H_
#define GFE_WS_CLIENT_H_

//#include "soapGFCP_WSProxy.h"
//#include "soapGFCP_USCORECEPortBindingProxy.h"
#include "soapGFCP_PortBindingProxy.h"
#include "soapH.h"

#include "../../Includes/concurrent_queue.h"
#include "../../XML_Handler/XML_Handler.h"
#include "../../Includes/GFEexception.h"
//#include "../../Includes/base64.h"
//#include "stdsoap2.h"
//#include "soapStub.h"


#include <string>
#include <stdio.h>

//enum GFE_WS_LoadLFB_Result
//{
//	LoadLFBResultSuccess,
//	LoadLFBResult_ErrorForwarder,
//	LoadLFBResult_ErrorClassifier,
//	LoadLFBResult_ErrorMapper
//};

//enum GFE_WS_AssignPort_Result
//{
//	AssignPortResultSuccess,
//	AssignPortResultFailure
//};

//enum GFE_WS_ResetPort_Result
//{
//	ResetPortResultSuccess,
//	ResetPortResultFailure
//};


/** @defgroup GFEWSClientG GFE WebService Client
 * @ingroup GFE_WSG
 *
 * \section GFE_WS_Client_main GFE WebService Client - Introduction
 *	The GFE_WS_Client is a module used for running with the GFE web server.
 *	This module supplies with the available functions by the GFE Web Server API, enabling basic GFE operations.
 *	Currently, the operations available are:<BR>
 *	1. \c LoadLFB - Load a LFB XML stream to the GFE system. <BR>
 *	2. \c AssignPort - Execute port assignment of a given frame type to a given port. <BR>
 *	3. \c ResetPort - Resets a given port's GFE tables.<BR>
 *
 *	\section GFE_WS_Client_func Creating configuration programs with the GFE WebService Client
 *	As this module offers all basic GFE operations, it can be used for creating new configuration programs.
 *	Currently, this module is operated by the GFE Control Panel Interface, yet using its functionality, developers can create any configuration interface.
 *	\subsection GFE_WS_Client_add_module Using the Module
 *	In order to use this module's functionality, one only needs to include it in his project of GFE configuration program, and create a \c GFE_WS_Client object, parameterized with the destination address and port (of the network processor). This object offers all needed functionality, as described above, and elaborated in the class' page.
 *	For example: <BR>
 *	\code
 *  // creating the object
 *  GFE_WS_Client client = GFE_WS_Client("http://123.45.67.89", 5555);
 *
 *  // loading XML contents to the GFE system
 *  int res = client.LoadLFB("forwarder",strm);
 *  if (res == LoadLFBResult_ErrorForwarder)
 *  	return err;
 *  // assigning port 1 with frame type 2
 *  res = client.AssignPort(1,2);
 *  \endcode
 * */

/** @defgroup GFE_WS_Client_Class GFE WS Client
 * @ingroup GFE_WSG
 * @{ */

/**  GFE_WS_Client Class - Instantiable.

 loads and executes a given script file
 */
class GFCP_WS_Client
{

public:
  GFCP_WS_Client(std::string, short int);
  GFCP_WS_Client(std::string, short int, std::string);
  GFCP_WS_Client(){};
  GFCP_WS_Client(const GFCP_WS_Client &c){};
  virtual ~GFCP_WS_Client();

  int loadLFB(LFB_structure *lfb);
  int sendFrameToCE(Frame *frm);
  void pushLFBToQueue(LFB_structure *lfbStruct);

private:
  void startQueueProcessingThread();
  static void *processQueue(void *_wsClient);

  GFCP_PortBindingProxy *proxy;
  std::string endpointURL;
  pthread_t threadPtr; //queue processing thread id
  concurrent_queue<LFB_structure *> egressSOAPQ; //thread safe frame queue

}; // class GFE_WS_Client

#endif /* GFE_WS_CLIENT_H_ */
