/*
 * GFCP_Server.cpp
 *
 *  Created on: May 31, 2011
 *      Author: yep
 */

#include "GFCP_Server.h"
#include "../../XML_Handler/XML_Handler.h"
//#include "../../GFCP_FE_Client/src/GFCP_WS_Client.h"
//#include "GFCP_WS.nsmap"
//#include "GFCP_USCOREFEPortBinding.nsmap"
#include "GFCP_PortBinding.nsmap"

using namespace std;

GFCP_Server::GFCP_Server(int port)
{
  //	gfcp_ws = new GFCP_WSService();
  gfcp_ws = new GFCP_PortBindingService();
  startLFBProcessingThread(port);

  /* run iterative server on port until fatal error */
  cout << "Started listening on port: " << port << endl;

}

GFCP_Server::GFCP_Server(int port, XML_Handler* xmlHandler)
{
  //	gfcp_ws = new GFCP_WSService();
  gfcp_ws = new GFCP_PortBindingService();
  gfcp_ws->setXMLHandler(xmlHandler);

  startLFBProcessingThread(port);

  /* run iterative server on port until fatal error */
  cout << "Started listening on port: " << port << endl;
}

//GFCP_Server::GFCP_Server(int port, GFCP_WS_Client* clientSOAP)
//{
//  //	gfcp_ws = new GFCP_WSService();
//  gfcp_ws = new GFCP_PortBindingService();
//  gfcp_ws->setClientSOAP(clientSOAP);
//
//  startQueueProcessingThread(port);
//
//  /* run iterative server on port until fatal error */
//  cout << "Started listening on port: " << port << endl;
//}

void GFCP_Server::startLFBProcessingThread(int port)
{
  wsStruct* ws = new wsStruct;
  ws->gfcp_ws = gfcp_ws;
  ws->port = port;

  threadPtr = 0;
  if (pthread_create(&threadPtr, NULL, runWS, (void *)ws))
    throw GFEexception("Failed to start new LLRE thread.");
}

void* GFCP_Server::runWS(void *wsPtr)
{
  wsStruct* ws = (wsStruct*)wsPtr;
  //	GFCP_WSService* gfcp =ws->gfcp_ws;
  GFCP_PortBindingService* gfcp = ws->gfcp_ws;
  if (gfcp->run(ws->port))
  {
    gfcp->soap_stream_fault(std::cerr);
  }
  delete ws;
  pthread_exit(0);
  return 0;
}

GFCP_Server::~GFCP_Server()
{
  gfcp_ws->stop();
  pthread_join(threadPtr, NULL);
  delete (gfcp_ws);
}

///// Web service operation 'query' (returns error code or SOAP_OK)
int GFCP_PortBindingService::queryRequest(ns1__dataRequest *ns1__queryRequest, ns1__dataResponse *ns1__queryResponse)
{
  return SOAP_OK;
}

/// Web service operation 'control' (returns error code or SOAP_OK)
int GFCP_PortBindingService::controlRequest(ns1__dataRequest *ns1__controlRequest, ns1__basicResponse *ns1__controlResponse)
{
  //  ns1__basicResponse res = new ns1__basicResponse();
  //  ns1__controlResponse->GFCPDst="dst";
  //  ns1__controlResponse->GFCPMessageID="1-1";
  //  ns1__controlResponse->GFCPSrc="src";
  //  ns1__controlResponse->description="desc";
  //  ns1__controlResponse->resultType=SOAP_OK;

  //  LFB_structure* lfbResponse;
  LFB_structure lfb;

  lfb.src = string(ns1__controlRequest->GFCPSrc);
  lfb.dst = string(ns1__controlRequest->GFCPDst);
  lfb.xml = string(ns1__controlRequest->data);
  lfb.transactionID = string(ns1__controlRequest->GFCPMessageID);

  cout << "\nGFCP_Server:Load LFB Initiated\n";
  cout << "transactionID: " << ns1__controlRequest->GFCPMessageID << "\nfile:\n"
      << ns1__controlRequest->data << endl << endl;

  xmlHanlder->Handle_LFB(lfb);

  ns1__controlResponse->GFCPDst=ns1__controlRequest->GFCPSrc;
  ns1__controlResponse->GFCPMessageID=ns1__controlRequest->GFCPMessageID;
  ns1__controlResponse->GFCPSrc=ns1__controlRequest->GFCPDst;
  vector<char> *desc = new vector<char>(lfb.description.size() + 1);
  std::copy(lfb.description.begin(), lfb.description.end(), desc->begin());
  ns1__controlResponse->description=&(*desc)[0];
  ns1__controlResponse->resultType=lfb.rsltType;

  cout << "\nGFCP_Server:Load LFB Finished\n";
  cout << "transactionID: " << ns1__controlRequest->GFCPMessageID
      << "\nresult:\n"<< lfb.rsltType
      << "\ndescription:\n"<< ns1__controlResponse->description << endl << endl;

  return SOAP_OK;
}

/// Web service operation 'frame' (returns error code or SOAP_OK)
int GFCP_PortBindingService::frameRequest(ns1__dataRequest *ns1__frameRequest, ns1__basicResponse *ns1__frameResponse)
{
  LFB_structure lfb;

  lfb.src = string(ns1__frameRequest->GFCPSrc);
  lfb.dst = string(ns1__frameRequest->GFCPDst);
  lfb.xml = string(ns1__frameRequest->data);
  lfb.transactionID = string(ns1__frameRequest->GFCPMessageID);

  cout << "\nGFCP_Server:Frame handling Initiated\n";
  cout << "transactionID: " << ns1__frameRequest->GFCPMessageID << "\nfile:\n"
      << ns1__frameRequest->data << endl << endl;

  //TODO Hook for frame handler
  lfb.rsltType = Success;    //Always report success


  ns1__frameResponse->GFCPDst=ns1__frameRequest->GFCPSrc;
  ns1__frameResponse->GFCPMessageID=ns1__frameRequest->GFCPMessageID;
  ns1__frameResponse->GFCPSrc=ns1__frameRequest->GFCPDst;
  vector<char> *desc = new vector<char>(lfb.description.size() + 1);
  std::copy(lfb.description.begin(), lfb.description.end(), desc->begin());
  ns1__frameResponse->description=&(*desc)[0];
  ns1__frameResponse->resultType=lfb.rsltType;

  cout << "\nGFCP_Server:Frame handling Finished\n";
  cout << "transactionID: " << ns1__frameRequest->GFCPMessageID
      << "\nresult:\n"<< lfb.rsltType
      << "\ndescription:\n"<< ns1__frameResponse->description << endl << endl;

  return SOAP_OK;
}

/// Web service operation 'notification' (returns error code or SOAP_OK)
int GFCP_PortBindingService::notificationRequest(ns1__dataRequest *ns1__notificationRequest, ns1__basicResponse *ns1__notificationResponse)
{
  return SOAP_OK;
}

/// Web service operation 'extension' (returns error code or SOAP_OK)
int GFCP_PortBindingService::extensionRequest(ns1__dataRequest *ns1__extensionRequest, ns1__dataResponse *ns1__extensionResponse)
{
  return SOAP_OK;
}

//int GFCP_WSService::loadLFB(unsigned long transactionID, int transactionType, char *data, int resultType, char *description, int *result)
//		unsigned long transactionID, int transactionType, char *data, int resultType, char *description, int *result)
//int GFCP_PortBindingService::loadLFB(ns1__loadLFB *ns1__loadLFB_, ns1__loadLFBResponse *ns1__loadLFBResponse_)
//{
//  ns1__loadLFB *req = ns1__loadLFB_;
//  LFB_structure* lfb = new LFB_structure;
//
//  lfb->src = new string(req->gfcpSrc);
//  lfb->dst = new string(req->gfcpDst);
//  lfb->xml = new string(req->data);
//  lfb->transactionID = req->transactionID;
//  lfb->lfbType = (TransactionType)req->transactionType;
//
//  lfb->rsltType = Success;
//  lfb->description = new string("GFCPdescription");
//  lfb->description = NULL;
//  lfb->rsltType = Not_Set;
//
//  switch (req->transactionType)
//  {
//    case 0:
//      lfb->lfbType = Classifier;
//      break;
//    case 1:
//      lfb->lfbType = Forwarder;
//      break;
//    case 2:
//      lfb->lfbType = Mapper;
//      break;
//    default:
//      //			lfb->lfbType=Classifier;
//      break;
//  }
//
//  xmlHanlder->pushLFBToQueue(lfb);
//
//  //TODO tmp debug only
//  //	clientSOAP->pushLFBToQueue(lfb);
//
//  cout << "\nGFCP_Server:Load LFB Initiated\n";
//  cout << "transactionID: " << req->transactionID << ", type:" << req->transactionType << endl << "file:"
//      << (req->data) << endl << endl;
//
//  ns1__loadLFBResponse_->return_ = SOAP_OK;
//  return SOAP_OK;
//}

