/****************************************************************************
 *
 * File Name: GFE.h
 *
 * General Description: General description of the file.
 *
 * Created By: Alex, Creation Date: 10/05/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

#ifndef GFE_H_
#define GFE_H_

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file LLRE.h
 *\brief LLRE header file.
 *
 * Full description of the file.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include <tr1/functional>
#include <string.h>
#include "../Dispatcher/Dispatcher.h"
#include "../LLRE/LLRE.h"
#include "../Network_Driver/NetworkDriver.h"
#include "../XML_Handler/XML_Handler.h"
#include "../Includes/concurrent_queue.h"
#include "../GFCP_FE_Server/src/GFCP_Server.h"
#include "../GFCP_FE_Client/src/GFCP_WS_Client.h"

/**
 * \brief Generic Forwarding Element (GFE) class.
 */
class GFE
{
public:

  //singleton
  static GFE& getInstance()
  {
    // The only instance
    // Guaranteed to be lazy initialized
    // Guaranteed that it will be destroyed correctly
    static GFE instance;
    return instance;
  }

  std::string startCapture(unsigned int interface, int num);
  std::string stopCapture(unsigned int interface);
  std::string StopAllActiveCapture();
  void printInitialTable();
  void printIDtoFWDTable();
  void printActionTable();
  void printInterfaces();
  void printXmlHandlerStatus();
  std::string loadClassifier(const char *LFB);
  std::string loadForwarder(const char *LFB);
  std::string loadMapper(const char *LFB);

  std::string disableEntry(unsigned int entryId);
  std::string enableEntry(unsigned int entryId);

  LLRE *getLlre();

private:
  // Private Constructor
  GFE();
  // Stop the compiler generating methods of copy the object
  GFE(GFE const& copy); // Not Implemented
  GFE& operator=(GFE const& copy); // Not Implemented

  virtual ~GFE();
  static void my_callback(u_char *args, const struct pcap_pkthdr* pkthdr, const u_char* packet);

  NetworkDriver *netdriver;
  LLRE *llre;
  Dispatcher *dispatcher;
  XML_Handler *xmlHandler;
  GFCP_Server *gfcp_server;
  GFCP_WS_Client *gfcp_client;
};

#endif /* GFE_H_ */
