/****************************************************************************
 *
 * File Name: Common.h
 *
 * General Description: Common definitions used in the GFE.
 *
 * Created By: Alex Creation Date: 01/01/2011
 *
 * Revisions And Updates:
 *
 * Date Updated By Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/
#ifndef COMMON_H_
#define COMMON_H_
/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file Common.h
 *\brief Common definitions used in the GFE.
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <vector>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h> /* includes net/ethernet.h */
#include <netinet/ip.h>       /* includes net/ethernet.h */
#include <net/ethernet.h>
#include <netinet/ether.h>
#include <boost/unordered/unordered_map.hpp>

#define BYTE unsigned char
#define MAX_FRAME_LEN 9000
#define CE_PORT 0x999
#define LLRE_PORT 0x1000

template<class T>
  std::ostream& operator <<(std::ostream& os, const std::vector<T>& v)
  {
    if (v.size() > 0)
    {
      os << "\n----------------";
      for (typename std::vector<T>::const_iterator ii = v.begin(); ii != v.end(); ++ii)
      {
        os << "\n" << *ii;
      }
      os << "\n----------------\n";
    }
    else
    {
      os << "\nvector is empty.\n";
    }
    return os;
  }

template<typename T>
  T StringToNumber(const std::string &Text)
  {
    std::stringstream ss(Text);
    T result;
    return ss >> result ? result : 0;
  }

template<typename T>
  T StringToHexNumber(const std::string &Text)
  {
    std::stringstream ss(Text);
    T result;
    return ss >> std::hex >> result ? result : 0;
  }

template<class T>
  bool more_vectors(const std::vector<T> &a, const std::vector<T> &b)
  {
    return a.size() > b.size();
  }

#endif /* COMMON_H_ */
