/*
 * maskedpatricia.h
 *
 *  Created on: Sep 23, 2011
 *      Author: b1t
 */

#ifndef MASKEDPATRICIA_H_
#define MASKEDPATRICIA_H_

//	typedef struct patricia_struct {
//		BIT_SELECT bit; /* what bit this node tests */
//		KEY_TYPE key; /* The value */
//		KEY_TYPE mask; /* The mask, for other than basic case */
//		KEY_TYPE common; /* bits common to all masks lower down */
//		int pos; /* depth - for discontiguous case */
//		struct patricia_struct *left, *right; /* child pointers */
//	} maskedpatriciaNode;
//
//	#define TEST_BIT(select, value) /* is the selected bit set? */
//	#define NEXT_BIT(select) /* generate the next sequential bit selector */
//	#define MAX_BIT /* The most significant bit selector */
//	#define HIGHER(select1, select2) /* is select1 for a node higher
//	in the tree than select2? */
//	/* replace with ‘‘pos1 < pos2’’ for discontiguous case */
//	#define EQ_KEY(key1, key2) /* are two keys equal? */
//	#define MASK_KEY(key, mask) /* apply the mask to the key */
//	#define SUBSET(key1, key2) /* all bits set in key1 are set in key2 */
#include <string.h> //for memmove
#include <stack>    //for stack
#include <exception>//for std::exception
#include <iostream> //for cout
#include <iomanip>  //for setw etc.
#include <vector>   //for nodes in a vect
#define MAX_KEY_SIZE 42

class maskedpatException : public std::exception
{
public:
  //! Constructs error
  maskedpatException(const char *what) :
    m_what(what)
  {
  }

  //! Gets human readable description of error.
  //! \return Pointer to null terminated description of the error.
  virtual const char *what() const throw ()
  {
    return m_what;
  }

private:
  const char *m_what;
};

class bytes
{
public:
  bytes()
  {
    byte = NULL;
    length = 0;
  }

  bytes(const bytes &other)
  {
    this->byte = NULL;
    *this = other;
  }

  bytes(unsigned char *_byte, unsigned int _length)
  {
    byte = _byte;
    length = _length;
  }

  virtual ~bytes()
  {
    //		if(this->byte != NULL)
    //			delete [] this->byte;
  }

  bytes& operator=(const bytes& b)
  {
    if (this->byte != NULL)
      delete[] this->byte;

    if (b.length > 0 && b.byte != NULL)
    {
      this->byte = new unsigned char[b.length];
      memmove(this->byte, b.byte, b.length);
      this->length = b.length;
    }
    else
    {
      this->byte = NULL;
      this->length = 0;
    }

    return *this;
  }

  bool operator==(const bytes & b) const
  {
    return (this->length == b.length) && (memcmp(this->byte, b.byte, this->length
        * sizeof(unsigned char)) == 0);
  }

  bool operator!=(const bytes & b) const
  {
    return (this->length != b.length) || (memcmp(this->byte, b.byte, this->length
        * sizeof(unsigned char)) != 0);
  }

  /**
   * mask compare, returns true if this mask has more higher bits set than b.
   *
   * 0xff >= 0xfe
   * 0xff >= 0xfe00
   */
  bool operator>=(const bytes & b) const
  {
    unsigned int len = ((this->length >= b.length) ? this->length : b.length);

    for (unsigned int i = 0; i < len; ++i)
    {
      if (this->byte[i] < b.byte[i])
        return false;
    }
    return true;
  }

  unsigned char *byte;
  unsigned int length;
};


template<class T>
  class maskedpatriciaNode
  {

    friend std::ostream& operator<<(std::ostream& output, const maskedpatriciaNode &node)
    {
      output << "[key=";
      printBytes(node.key);
      output << ", mask=";
      printBytes(node.mask);
      output << ", bit_select=" << std::dec <<node.bit_select << ", value=";
      output << node.value<<"]"<<std::endl;
      output << std::endl;
      return output;
    }

    friend std::ostream& operator<<(std::ostream& output, const maskedpatriciaNode *node)
    {
      output << "[key=";
      printBytes(node->key);
      output << ", mask=";
      printBytes(node->mask);
      output << ", bit_select=" << std::dec <<node->bit_select << ", value=";
      output << node->value<<"]"<<std::endl;
      output << std::endl;
      return output;
    }

  public:
    maskedpatriciaNode();
    maskedpatriciaNode(bytes &key, bytes &mask, T value, unsigned int bit_select,
                       maskedpatriciaNode<T> *left, maskedpatriciaNode<T> *right);
    virtual ~maskedpatriciaNode();

    T getValue();
    void setValue(T value);
    bytes getKey();
    maskedpatriciaNode<T> *getLeft();
    maskedpatriciaNode<T> *getRight();
    static void printBytes(const bytes &b);

    //TODO temp make private with getters and setters
    bytes key;
    bytes mask;
    T value;
    unsigned int bit_select;
    maskedpatriciaNode<T> *left;
    maskedpatriciaNode<T> *right;

  private:
    void init(bytes &key, bytes &mask, T value, unsigned int bit_select,
              maskedpatriciaNode<T> *left, maskedpatriciaNode<T> *right);

  };

template<class T>
  class maskedpatricia
  {
  public:
    maskedpatricia();
    virtual ~maskedpatricia();

    virtual maskedpatriciaNode<T> *insert(bytes &key, bytes &mask, T value);
    virtual bool exists(bytes &key, bytes &mask);
    virtual T find(bytes &key);
    virtual T* findReference(bytes &key);
    virtual bool remove(bytes &key, bytes &mask);

    //	private:
    void freeTree(maskedpatriciaNode<T> *node);

    void clearTree();

    void clearTreeRecursive(maskedpatriciaNode<T> *node);

    //	#define TEST_BIT(select, value) /* is the selected bit set? */
    bool isSelectedBitSet(bytes &value, unsigned int select);

    //	#define NEXT_BIT(select) /* generate the next sequential bit selector */
    void nextSelectBit(unsigned int select);

    //	#define MASK_KEY(key, mask) /* apply the mask to the key */
    void applyMaskToKey(bytes &key, bytes &mask);

    //	#define SUBSET(key1, key2) /* all bits set in key1 are set in key2 */
    bool subsetBits(bytes &key1, bytes &key2);

    maskedpatriciaNode<T> *find_up(maskedpatriciaNode<T> *node,
                                   unsigned int select);

    void Walk_Default(maskedpatriciaNode<T> *trav, bool installOrFree,
                      maskedpatriciaNode<T> *toAdd);

    void printTree();

    void printTreeRecursive(maskedpatriciaNode<T> *node);

    maskedpatriciaNode<T> *patricia_insert(bytes &key, bytes &mask, T value);

    void fillVectorOfNodes(std::vector<maskedpatriciaNode<T>*> &vect);

    void addNodeToVectorOfNodes(maskedpatriciaNode<T> *node, std::vector<
        maskedpatriciaNode<T>*> &vect);

//    maskedpatriciaNode<T> *insert_disc_mask(bytes &key, bytes &mask,
//                                                       unsigned int recommend, T value);

    maskedpatriciaNode<T> *root;
  };

template<class T>
  maskedpatriciaNode<T>::maskedpatriciaNode()
  {
    unsigned char *byte = new unsigned char[MAX_KEY_SIZE];
    //    byte[0] = 0x00;
    memset(&byte[0], 0x00, MAX_KEY_SIZE);
    bytes b(byte, MAX_KEY_SIZE);

    unsigned int bs;
    bs = 0;
    init(b, b, T(), bs, this, this);

    delete[] byte;
  }

template<class T>
  maskedpatriciaNode<T>::maskedpatriciaNode(bytes &key, bytes &mask, T value,
                                            unsigned int bit_select,
                                            maskedpatriciaNode<T> *left,
                                            maskedpatriciaNode<T> *right)
  {
    init(key, mask, value, bit_select, left, right);
  }

template<class T>
  void maskedpatriciaNode<T>::init(bytes &key, bytes &mask, T value,
                                   unsigned int bit_select,
                                   maskedpatriciaNode<T> *left,
                                   maskedpatriciaNode<T> *right)
  {
    this->key = key;
    this->mask = mask;
    this->value = value;
    this->bit_select = bit_select;
    this->left = left;
    this->right = right;
  }

template<class T>
  maskedpatriciaNode<T>::~maskedpatriciaNode()
  {
    if (key.byte != NULL)
      delete[] key.byte;
    if (mask.byte != NULL)
      delete[] mask.byte;
  }

template<class T>
  T maskedpatriciaNode<T>::getValue()
  {
    return value;
  }

template<class T>
  void maskedpatriciaNode<T>::setValue(T value)
  {
    this->value = value;
  }

template<class T>
  bytes maskedpatriciaNode<T>::getKey()
  {
    return key;
  }

template<class T>
  maskedpatriciaNode<T> *maskedpatriciaNode<T>::getLeft()
  {
    return left;
  }

template<class T>
  maskedpatriciaNode<T> *maskedpatriciaNode<T>::getRight()
  {
    return right;
  }

//TODO add namespace
template<class T>
  void maskedpatriciaNode<T>::printBytes(const bytes &b)
  {
    if (b.byte != NULL)
    {
      std::cout << "|";
      for (unsigned int i = 0; i < b.length; ++i)
      {
        if (i != 0 && i % 8 == 0)
          std::cout << "\n|";

        std::cout << std::internal << std::hex << std::setw(4)
            << std::setfill('0') << (unsigned short *)b.byte[i] << "|";
      }
    }
    else
      std::cout << "No value defined";
  }

template<class T>
  maskedpatricia<T>::maskedpatricia()
  {
    root = new maskedpatriciaNode<T> ();
  }

template<class T>
  maskedpatricia<T>::~maskedpatricia()
  {
    freeTree(root);
  }

template<class T>
  void maskedpatricia<T>::freeTree(maskedpatriciaNode<T> *node)
  {
    if (node == NULL)
      return;

    if (node->bit_select < node->left->bit_select)
      freeTree(node->left);

    if (node->bit_select < node->right->bit_select)
      freeTree(node->right);

    delete node;
  }

template<class T>
  void maskedpatricia<T>::clearTree()
  {
    clearTreeRecursive(root);
  }

template<class T>
  void maskedpatricia<T>::clearTreeRecursive(maskedpatriciaNode<T> *node)
  {
    if (node == NULL)
      return;

    if (node->bit_select < node->left->bit_select)
      freeTree(node->left);

    if (node->bit_select < node->right->bit_select)
      freeTree(node->right);

    if (node != root)
      delete node;
    else
    {
      node->left = node;
      node->right = node;
    }
  }

template<class T>
  void maskedpatricia<T>::fillVectorOfNodes(
                                            std::vector<maskedpatriciaNode<T>*> &vect)
  {
    addNodeToVectorOfNodes(root, vect);
  }

template<class T>
  void maskedpatricia<T>::addNodeToVectorOfNodes(
                                                 maskedpatriciaNode<T> *node,
                                                 std::vector<
                                                     maskedpatriciaNode<T>*> &vect)
  {
    if (node == NULL)
      return;

    if (node != root)
      vect.push_back(node);

    if (node->bit_select < node->left->bit_select)
      addNodeToVectorOfNodes(node->left, vect);

    if (node->bit_select < node->right->bit_select)
      addNodeToVectorOfNodes(node->right, vect);
  }

template<class T>
  maskedpatriciaNode<T> *maskedpatricia<T>::insert(bytes &key, bytes &mask,
                                                   T value)
  {
    maskedpatriciaNode<T> *parent, *trav, *next, *pp, *n, *tmp;
    unsigned int try_bit = 0;

    ///////////////////////////////////////////////////////////////////////////////////////////
    //                                          Lookup                                       //
    ///////////////////////////////////////////////////////////////////////////////////////////
    trav = root;
    do
    {
      parent = trav;
      trav = (isSelectedBitSet(key, trav->bit_select)) ? trav->right : trav->left;

    } while (parent->bit_select < trav->bit_select);

    ///////////////////////////////////////////////////////////////////////////////////////////
    //                                          Found match                                  //
    ///////////////////////////////////////////////////////////////////////////////////////////
    /*
     * If the new value is actually in the tree, we must
     * determine if masks have changed, and cope with that.
     */
    if (trav->key == key)
    {
      /*
       * test for the simple case of identical or extending mask
       */
      if (mask >= trav->mask)
      {
        trav->mask = mask;
        return (trav);
      }
      /*
       * The mask is becoming shorter, check to see if
       * the node needs to be moved up.
       */
      next = root;
      bytes maskedKey;
      do
      {
        /* save the parent */
        parent = next;
        next = (isSelectedBitSet(key, next->bit_select)) ? next->right
            : next->left;

        maskedKey = next->key;
        applyMaskToKey(maskedKey, mask);

      } while (next != trav && maskedKey != key);

      if (next == trav)
      {
        /* there is no need to move it */
        trav->mask = mask;
        return (trav);
      }
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    //                                     Locate Insertion                                  //
    ///////////////////////////////////////////////////////////////////////////////////////////
    else
    {/* new value NOT in tree */
      //SET_MAX_BIT(try_bit);
      try_bit = 0;
      /*
       * Find the first bit by which the new key differs from
       * what was found. There must be one.
       */

      while (isSelectedBitSet(key, try_bit) == isSelectedBitSet(trav->key,
                                                                try_bit))
        try_bit++;

      //node keys are of different length, and still diff. bit was not found,
      //then look for highest set bit in the longer key
      if(isSelectedBitSet(key, try_bit-1) == isSelectedBitSet(trav->key, try_bit-1))
      {
        while(try_bit / 8 < key.length &&
              !isSelectedBitSet(key, try_bit))
          try_bit++;

        if(try_bit / 8 == key.length)//value is zero, need to go 1 step back to be in range
          try_bit--;
      }
      /*
       * Now check whether, on the basis of this bit,
       * the new node belongs somewhere else in the tree.
       *
       * In addition to the standard checks, we test to see if
       * there is a node which must be below the new one, defining
       * an insert position just above it.
       */
      trav = root;
      bytes maskedKey;
      do
      {
        /* as in the lookup, save the parent */
        parent = trav;
        trav = (isSelectedBitSet(key, trav->bit_select)) ? trav->right
            : trav->left;

        maskedKey = trav->key;
        applyMaskToKey(maskedKey, mask);

      } while (trav->bit_select < try_bit && parent->bit_select
          < trav->bit_select && maskedKey != key);
      delete[] maskedKey.byte;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    //                                     Tree Rebuild Part 1                               //
    ///////////////////////////////////////////////////////////////////////////////////////////
    /* --------------------------------------------------------------------------------------------------*/
    /*
     * Now test to see if we are doing an unusual middle insert.
     * If so, we must remove the lower subtree, add the node, and
     * then re-insert the subtree.
     */
    bytes maskedKey(trav->key);
    applyMaskToKey(maskedKey, mask);

    if (maskedKey == key)
    {
      delete[] maskedKey.byte;
      /*
       * Find where the pointer from parent would have pointed,
       * if there had been no subtree under it.
       */
      if (isSelectedBitSet(key, parent->bit_select))
      {
        //TODO unclear ???

        /* we are in the right subtree */
        if (isSelectedBitSet(parent->key, parent->bit_select))
          /* parent’s right would have pointed to parent */
          pp = parent;
        else
          //pp = find_up(parent->right, parent->bit_select);
          pp = parent->right;
        /* The routine find_up is used otherwise, to find out where the
         up-pointer points. It recursively searches the tree looking for that pointer */
      }
      else
      {
        //TODO unclear ???

        /* we are in the left subtree */
        if (!isSelectedBitSet(parent->key, parent->bit_select))
          /* parent’s left would have been parent */
          pp = parent;
        else
          //pp = find_up(parent->left, parent->bit_select);
          pp = parent->left;
      }
      /* determine which bit we differ from pp on */
      //SET_MAX_BIT(try_bit);
      try_bit = 0;
      while (isSelectedBitSet(key, try_bit) == isSelectedBitSet(pp->key, try_bit))
        try_bit++;


      //node keys are of different length, and still diff. bit was not found,
      //then look for highest set bit in the longer key
      if(isSelectedBitSet(key, try_bit-1) == isSelectedBitSet(pp->key, try_bit-1))
      {
        while(try_bit / 8 < key.length &&
              !isSelectedBitSet(key, try_bit))
          try_bit++;

        if(try_bit / 8 == key.length)//value is zero, need to go 1 step back to be in range
          try_bit--;
      }

      /* Create new node */
      if (isSelectedBitSet(key, try_bit))
      {
        n = new maskedpatriciaNode<T> (key, mask, value, try_bit, pp, NULL);
        n->right = n;
      }
      else
      {
        n = new maskedpatriciaNode<T> (key, mask, value, try_bit, NULL, pp);
        n->left = n;
      }

      ///////////////////////////////////////////////////////////////////////////////////////////
      //                                     Tree Rebuild Part 2                               //
      ///////////////////////////////////////////////////////////////////////////////////////////
      /*
       * 	Having built the new node, the code checks to see if there is a subtree in place.
       *  If so, the nodes in it are copied into the tree under the new node, one at a time.
       *  By doing the traversal in depth first order, no additional recursion should be required.
       *  Then, one simply inserts the new subtree into the tree, and gets rid of the old nodes.
       *
       * 	Determine which side of p we go on,
       * 	And if there is a subtree there,
       * 	Rebuild it under the new node.
       */
      if (!isSelectedBitSet(key, parent->bit_select) && parent->left->bit_select
          >= parent->bit_select)
      {
        ////			Walk_Default (trav, Install_Node, n);
        //			Walk_Default (trav, true, n);
        tmp = parent->left;
        n->right = tmp;
        tmp->left = n;
        //update bit select in the moved down node
        while (isSelectedBitSet(n->key, try_bit) == isSelectedBitSet(tmp->key,
                                                                     try_bit))
          try_bit++;
        tmp->bit_select = try_bit;

      }
      else if (!isSelectedBitSet(key, parent->bit_select)
          && parent->right->bit_select >= parent->bit_select)
      {
        ////			Walk_Default (trav, Install_Node, n);
        //			Walk_Default (trav, true, n);
        tmp = parent->right;
        n->left = tmp;
        tmp->right = n;
        //update bit select in the moved down node
        while (isSelectedBitSet(n->key, try_bit) == isSelectedBitSet(tmp->key,
                                                                     try_bit))
          try_bit++;
        tmp->bit_select = try_bit;
      }
      /*
       * Hook-up the rebuilt tree
       */
      if (isSelectedBitSet(key, parent->bit_select))
        parent->right = n;
      else
        parent->left = n;
      /*
       * Free all nodes in the old subtree trav.
       */
      //		Walk_Default (trav, Free_Node, NULL);
      //		Walk_Default (trav, false, NULL);

      return (n);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    //                                    Normal insert                                      //
    ///////////////////////////////////////////////////////////////////////////////////////////
    else
    { /* normal case */
      /*
       * now insert the new node between parent and trav
       */

      delete[] maskedKey.byte;

      if (isSelectedBitSet(key, try_bit))
      {
        next = new maskedpatriciaNode<T> (key, mask, value, try_bit, trav, NULL);
        next->right = next;
      }
      else
      {
        next = new maskedpatriciaNode<T> (key, mask, value, try_bit, NULL, trav);
        next->left = next;
      }

      if (isSelectedBitSet(key, parent->bit_select))
        parent->right = next;
      else
        parent->left = next;

      return (next);
    }
  }

#ifdef nn
//PNODE *P_Insert (addr, mask, ptree, recommend)
//KEY_TYPE addr;
//KEY_TYPE mask;
//patricia_node *ptree;
//BIT_SELECT recommend;

/*
 * Discontigous masks
 */
template<class T>
  maskedpatriciaNode<T> *maskedpatricia<T>::insert_disc_mask(bytes &key, bytes &mask,
                                                   unsigned int recommend, T value)
  {
    maskedpatriciaNode<T> *n, *p, *pp, *t, *up, *pc, *splice;
    unsigned int bit = 0;
    bytes prev_common;
    std::stack<maskedpatriciaNode<T> *> pm_stck;
    int clone_parent;
    /*
    * Apply the mask to the address
    * This should be an irrelevant operation, but protects against
    * incorrect calls where the address has bits set outside the mask.
    */
    applyMaskToKey(key, mask);
    /*
    * Lookup the address.
    */
    t = root;
    bytes maskedKey;
    do {
      p = t;
//      PNODE_PUSH(p, pm_stack_ptr);
      stck.push(p);
      t = (isSelectedBitSet(key, t->bit_select)) ? t->right : t->left;

      maskedKey = t->key;
      applyMaskToKey(maskedKey, mask);
    } while ( isSelectedBitSet(mask, t->bit_select) &&
              p->pos < t->pos /* t is deeper than p */ &&
              maskedKey != key);
    if ((key == t->key) && isSelectedBitSet(mask, t->bit_select))
    {
      /*
      * The new mask is usable without re-ordering anything.
      * It may change ’common’ higher in the tree, but by the descent
      * procedure, it will not invalidate any checks.
      */
      t->mask = mask;
      prev_common = t->common;
      t->common = mask;
      if (t->left->pos > t->pos) /* There is a left subtree */
        applyMaskToKey(t->common, t->left->common);

      if (t->right->pos > t->pos) /* There is a right subtree */
        applyMaskToKey(t->common, t->right->common);

        n = t; /* for return */
    } else
    {
      if (t->pos <= p->pos) /* t is higher in the tree than p */
        up = t;
      else /* we need to find the up pointer */
      {
        if (isSelectedBitSet(key, p->bit_select)) /* we belong on p’s right */
        {
          if (isSelectedBitSet(p->key, p->bit_select)) /* which is p */
          {
            up = p;
          } else {
            up = Find_Up (p->right, p->pos);
          }
        } else /* we go on the left */ {
          if (!isSelectedBitSet(p->key, p->bit_select)) /* which is p */ {
            up = p;
          } else {
            up = Find_Up (p->left, p->pos);
          }
        }
      }
    /*
    * Now find the most significant bit on which we differ
    * from the located node.
    * Significance is defined by the tree of common masks.
    */
    PNODE_STACKBOT(pm_trav, pm_stack);
    while (!PNODE_STACKTOP(pm_trav, pm_stack_ptr)) {
      prev_common = MASK_KEY(mask, PNODE_STACKGET(pm_trav)->common);
      if (!EQ_KEY(MASK_KEY(key, prev_common),MASK_KEY(up->key, prev_common)))
        /* there is a bit in this mask on which we differ */
        break;
      PNODE_STACKNEXT(pm_trav);
    }
    if (!PNODE_STACKTOP(pm_trav, pm_stack_ptr)) {
      prev_common = PNODE_STACKGET(pm_trav)->common;
      clone_parent = 0;
    } else
    {
      prev_common = mask; /* use our mask */
      if (t->pm_pos > p->pm_pos) /* there is a down pointer */ {
        prev_common = MASK_KEY(prev_common, t->common);
        clone_parent = 0;
        /* now check whether we can use this */
        if (EQ_KEY(MASK_KEY(key, prev_common),MASK_KEY(up->key, prev_common))) {
          prev_common = mask;
          clone_parent = 1;
        } else {
          clone_parent = 1; /* probably irrelevant */
        }
      }
      /*
      * Having found a mask under which key and up->pm_addr differ,
      * now select a bit. We use the easiest one to find.
      */
      /*
      * We will first see if the recommended bit can be used, since
      * That presumably works well with the subtree.
      * If it is in the mask we have selected, then we can check to
      * see if it distinguishes the values.
      *
      * The recommended bit is used when re-inserting nodes, to try
      * to use the same bits as before, reducing the occurence of
      * recursive rebuilds.
      * These may occur because the discontiguous mask inferrence may
      * Take two steps to reach the final configuration.
      */
      if (isSelectedBitSet(prev_common, recommend) &&
      (isSelectedBitSet(key, recommend) != isSelectedBitSet(up->key, recommend))) {
        bit = recommend;
      } else {
        /* just find the first distinguishing bit */
        firstbit(&bit, prev_common); /*find first set bit */
        while (isSelectedBitSet(key, bit) == isSelectedBitSet(up->key, bit)) {
          nextbit(&bit, prev_common); /* find next set bit */
        }
      }
      n = new maskedpatriciaNode<T>(key, mask, value, bit);
      /*
      * Now figure out where to put the node.
      * Put it either under p, or above the first node whose
      * tested bit is not in prev_common.
      * prev_common defines the significance of our bit.
      */
      pp = ptree;
      do {
        t = pp;
        pp = isSelectedBitSet(key, pp->bit_select) ? pp->right : pp->left;
      } while (t != p /* not to bottom of our space */ &&
              isSelectedBitSet(prev_common, pp->bit_select));
      /* now insert between t and pp */
      n->pm_pos = t->pm_pos + 1;
      /*
      * This preserves the common information we needed above,
      * for future use.
      */
      if (pp->pos > t->pos&& !clone_parent)
        APPLY_MASK(n->common, pp->common);
      if (pp->pos <= t->pos /* leaf insertion */ ||
          t != p /* simple significance */) {
        n->left = isSelectedBitSet(key, n->bit_select) ? pp : n;
        n->right = isSelectedBitSet(key, n->bit_select) ? n : pp;
        splice = n;
        if (pp->pos > t->pos /* middle insertion */)
          /* then increase the subtree depth */
          Walk_PostFix (pp, Increase_pos);
      } else { /* there is a subtree, which we were forced above */
        /* we already found the up pointer from this tree */
        n->pm_left = isSelectedBitSet(key, n->bit_select) ? up : n;
        n->pm_right = isSelectedBitSet(key, n->bit_select) ? n : up;
        if (clone_parent) {
          pc = Alloc_Node();
          *pc = *t;
          pc->common = MASK_KEY(pc->common, n->common);
          if (pc->left == pp) {
            pc->left = n;
            Walk_Default(pp, Install_Node, pc);
            splice = pc->left;
          } else {
            pc->right = n;
            Walk_Default(pp, Install_Node, pc);
            splice = pc->right;
          }
          /*
          * The up pointer may have been moved and
          * Replaced with a pointer to pc. Fix it.
          */
          (void *)replace_ptr(splice, pc, up);
          Free_Node(pc);
        } else {
          Walk_Default(pp, Install_Node, n);
          splice = n;
        }
      }
      /*
      * while key may not be the address in splice,
      * it does select the correct side.
      */
      if (isSelectedBitSet(key, t->bit_select))
        t->right = splice;
      else
        t->left = splice;

      if (pp->pos > t->pos && t == p)
        Walk_Default(pp, Free_Node, NULL);

      prev_common = t->common;
      APPLY_MASK(t->common, n->common);
      /* clear all the lower stuff off the stack */
      while (!PNODE_STACKEMPTY(pm_stack_ptr, pm_stack)) {
        p = PNODE_POP(pm_stack_ptr);
        if (p == t) /* we have cleared t off the stack */
          break;
      }
    }
    if (!EQ_KEY(prev_common, t->common) &&
        !PNODE_STACKEMPTY(pm_stack_ptr, pm_stack)) {
      /*
      * common bits have changed,
      * so propogate the changes up the tree
      */
      if (SUBSET(t->common, prev_common)) { /* bits have shrunk */
        do {
          p = PNODE_POP(pm_stack_ptr);
          prev_common = p->common;
          p->common = MASK_KEY(p->common, t->common);
          t = p;
        } while (!PNODE_STACKEMPTY(pm_stack_ptr, pm_stack) &&
                !EQ_KEY(prev_common, t->common));
      } else { /* bits have grown */
        do {
          p = PNODE_POP(pm_stack_ptr);
          prev_common = p->common;
          p->common = p->mask;
          p->common = MASK_KEY(p->common, t->common);
          if (isSelectedBitSet(t->key, p->bit_select)) { /* p is on right */
            if (p->left->pos > p->pos) /* left is down */
              p->common = MASK_KEY(p->common, p->left->common);
          } else if (p->right->pos > p->pos) /* right is down */
            p->common = MASK_KEY(p->common, p->right->common);

          t = p;
        } while (!PNODE_STACKEMPTY(pm_stack_ptr, pm_stack) &&
                !EQ_KEY(prev_common, t->common));
        }
    }

    return (n); /* success */
  }
#endif

/*
 * without mask
 */
template<class T>
  maskedpatriciaNode<T> *maskedpatricia<T>::patricia_insert(bytes &key,
                                                            bytes &mask, T value)
  {
    maskedpatriciaNode<T> *parent, *trav, *new_node;
    unsigned int try_bit;

    //	trav = patricia_find(key, mask, root);
    trav = root;
    do
    {
      /*
       * To detect the termination, we must find a pointer that
       * does not descend in the tree. This requires saving
       * the current node before proceeding down in the direction
       * selected by the tested bit.
       */
      parent = trav;
      if (isSelectedBitSet(key, trav->bit_select))
        trav = trav->right;
      else
        trav = trav->left;
    } while (parent->bit_select < trav->bit_select);

    if (trav->key == key)
      return (trav);

    //	SET_MAX_BIT(try_bit);
    try_bit = 0;
    /*
     * Find the first bit by which the new key differs from
     * what was found. There must be one.
     */
    while (isSelectedBitSet(key, try_bit) == isSelectedBitSet(trav->key, try_bit))
      try_bit++;
    /*
     * now check whether, on the basis of this bit,
     * the new node belongs somewhere else in the tree.
     */
    trav = root;
    do
    {
      /* as in the lookup, save the parent */
      parent = trav;
      if (isSelectedBitSet(key, trav->bit_select))
        trav = trav->right;
      else
        trav = trav->left;
    } while (trav->bit_select < try_bit && parent->bit_select < trav->bit_select);
    /*
     * now insert the new node between parent and trav
     */
    new_node = new maskedpatriciaNode<T> (key, mask, value, try_bit, NULL, NULL);

    if (isSelectedBitSet(key, try_bit))
    {
      new_node->left = trav;
      new_node->right = new_node;
    }
    else
    {
      new_node->left = new_node;
      new_node->right = trav;
    }

    if (isSelectedBitSet(key, parent->bit_select))
      parent->right = new_node;
    else
      parent->left = new_node;
    return (new_node);
  }

template<class T>
  void maskedpatricia<T>::Walk_Default(maskedpatriciaNode<T> *trav,
                                       bool installOrFree,
                                       maskedpatriciaNode<T> *toAdd)
  {

  }

/* The routine find_up is used otherwise, to find out where the
 up-pointer points. It recursively searches the tree looking for that pointer */
template<class T>
  maskedpatriciaNode<T> *maskedpatricia<T>::find_up(maskedpatriciaNode<T> *node,
                                                    unsigned intselect)
  {
    return NULL;
  }

template<class T>
  bool maskedpatricia<T>::exists(bytes &key, bytes &mask)
  {
    return true;
  }

template<class T>
  T maskedpatricia<T>::find(bytes &key)
  {
    maskedpatriciaNode<T> *parent, *trav;
    /*
     * Standard Patricia search, and
     * remember the nodes visited along the way.
     */
    std::stack<maskedpatriciaNode<T> *> stck;
    trav = root;
    do
    {
      /* save the parent to detect the up-pointer  */
      parent = trav;
      /* push what we have passed */
      stck.push(parent);
      trav = (isSelectedBitSet(key, trav->bit_select)) ? trav->right : trav->left;

    } while (parent->bit_select < trav->bit_select);
    stck.push(trav); /* so it is tested first */
    /*
     * Search up the tree for the best match
     */
    bytes maskedKey;
    while (!stck.empty())
    {
      trav = stck.top();
      stck.pop();
      maskedKey = key;
      applyMaskToKey(maskedKey, trav->mask);
      if (maskedKey == trav->key)
      {
        delete[] maskedKey.byte;
        return (trav->value);
      }
    }
    delete[] maskedKey.byte;

    return root->value;
  }

template<class T>
  T* maskedpatricia<T>::findReference(bytes &key)
  {
    maskedpatriciaNode<T> *parent, *trav;
    /*
     * Standard Patricia search, and
     * remember the nodes visited along the way.
     */
    std::stack<maskedpatriciaNode<T> *> stck;
    trav = root;
    do
    {
      /* save the parent to detect the up-pointer  */
      parent = trav;
      /* push what we have passed */
      stck.push(parent);
      trav = (isSelectedBitSet(key, trav->bit_select)) ? trav->right : trav->left;

    } while (parent->bit_select < trav->bit_select);
    stck.push(trav); /* so it is tested first */
    /*
     * Search up the tree for the best match
     */
    bytes maskedKey;
    while (!stck.empty())
    {
      trav = stck.top();
      stck.pop();
      maskedKey = key;
      applyMaskToKey(maskedKey, trav->mask);
      if (maskedKey == trav->key)
      {
        delete[] maskedKey.byte;
        return (&trav->value);
      }
    }
    delete[] maskedKey.byte;

    return &root->value;
  }

template<class T>
  bool maskedpatricia<T>::remove(bytes &key, bytes &mask)
  {
    return true;
  }

template<class T>
  bool maskedpatricia<T>::isSelectedBitSet(bytes &value, unsigned int select)
  {
    unsigned int byteNum = select / 8;
    short bitNum = select % 8;
    if (value.length < byteNum)
      throw maskedpatException("Selected bit is out of range.\n");

    if (value.byte == NULL)
      throw maskedpatException("Encountered NULL value.\n");

    unsigned char *bytePtr = value.byte;
    bytePtr += byteNum;
    unsigned char mask = 0x80 >> bitNum;

    //TODO remove shifts and replace with > 0
    int testedBit = ((*bytePtr) & mask) << bitNum;

    return (testedBit == 0) ? false : true;
  }

template<class T>
  void maskedpatricia<T>::nextSelectBit(unsigned int select)
  {
    //	select.byteNum += (select.bitNum + 1) / 8;
    //	select.bitNum = (select.bitNum + 1) % 8;
    select++;
  }

template<class T>
  void maskedpatricia<T>::applyMaskToKey(bytes &key, bytes &mask)
  {
    //	if(key.length !=  mask.length)
    //		throw maskedpatException("Key and Mask have different size.\n");
    //
    //	for (unsigned int i = 0; i < key.length; ++i) {
    //		key.byte[i] = key.byte[i] & mask.byte[i];
    //	}

    unsigned int len = ((key.length >= mask.length) ? key.length : mask.length);

    for (unsigned int i = 0; i < len; ++i)
    {
      key.byte[i] = key.byte[i] & mask.byte[i];
    }
  }

template<class T>
  bool maskedpatricia<T>::subsetBits(bytes &key1, bytes &key2)
  {
    return true;
  }

template<class T>
  void maskedpatricia<T>::printTree()
  {
    if (root != NULL)
    {
      std::cout << "\n--------------------------\n";
      printTreeRecursive(root);
      std::cout << "--------------------------\n";
    }
  }

template<class T>
  void maskedpatricia<T>::printTreeRecursive(maskedpatriciaNode<T> *node)
  {
    if (node != NULL)
    {
      //print current node
      if (node != root)
        std::cout << *node;

      //walk left
      if (node->bit_select < node->left->bit_select)
      {
        std::cout << "Left:  ";
        printTreeRecursive(node->left);
      }

      //walk right
      if (node->bit_select < node->right->bit_select)
      {
        std::cout << "Right: ";
        printTreeRecursive(node->right);
      }
    }
  }

#endif /* MASKEDPATRICIA_H_ */
