/****************************************************************************
 *
 * File Name: Action_Add.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 14/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Action_Add.cpp
 *\brief Implements the Action_Add Class.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Add.h"

using namespace std;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
Action_Add::Action_Add()
{
  position.setOffset(0);
  position.setLength(0);
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Add::Action_Add(unsigned int offset, unsigned int length)
{
  position.setOffset(offset);
  position.setLength(length);
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other - reference to the received Action_Add, that is going to be copied.
 */
/****************************************************************************/
Action_Add::Action_Add(const Action_Add &other)
{
  this->position.setOffset(other.position.getOffset());
  this->position.setLength(other.position.getLength());
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
Action_Add::~Action_Add()
{
}

/****************************************************************************/
//! Perform this action on the received frame.
/*! Perform this action on the received frame.
 *
 * @param frm - frame structure to perform the action on.
 * @return a string that describes the result of the operation.
 */
/****************************************************************************/
string Action_Add::performAction(Frame *frm)
{
  if (position.getOffset() + frm->length > MAX_FRAME_LEN)
    return "Invalid offset - Maximum frame length exceeded";

  BYTE *newFramePtr = new BYTE[frm->length + position.getLength()];

  memmove(newFramePtr, frm->frame, position.getOffset());

  memmove(&newFramePtr[position.getOffset() + position.getLength()], &(frm->frame)[position.getOffset()], frm->length
      - position.getOffset());

  delete[] frm->frame;
  frm->frame = newFramePtr;
  frm->length += position.getLength();
  return "DONE";
}

/****************************************************************************/
//! Prints action details.
/*! Prints action details.
 * @param ostream for cascaded output.
 */
/****************************************************************************/
void Action_Add::printAction(ostream &) const
{
  cout << "Action_Add: offset=" << dec << position.getOffset() << ", length=" << dec << position.getLength();
}
