/****************************************************************************
 *
 * File Name: Action_Arithmetic.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 14/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Action_Arithmetic.cpp
 *\brief Implements the Action_Arithmetic Class.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Arithmetic.h"

using namespace std;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
Action_Arithmetic::Action_Arithmetic()
{
  //TODO change BYTE * to actual values
  dstPosition.setOffset(0);
  dstPosition.setLength(0);
  operation = -1;
  operand = new BYTE[1];
  operand[0] = 0xff;
  mask = new BYTE[1];
  mask[0] = 0xff;
  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Arithmetic::Action_Arithmetic(unsigned int dstOffset, unsigned int length, int _operation,
                                     unsigned char *_operand, unsigned char *_mask)
{
  dstPosition.setOffset(dstOffset);
  dstPosition.setLength(length);
  operation = _operation;

  if (_operand == NULL)
  {
    operand = NULL;
  }
  else
  {
    operand = new BYTE[length];
    memcpy(operand, _operand, length);
  }

  if (_mask == NULL)
  {
    mask = NULL;
  }
  else
  {
    mask = new BYTE[length];
    memcpy(mask, _mask, length);
  }
  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Arithmetic::Action_Arithmetic(unsigned int dstOffset, unsigned int length, int _operation,
                                     unsigned char *_operand, unsigned char *_mask, unsigned int chksumOffset,
                                     unsigned int chksumLength, bool _isPerformAction)
{
  dstPosition.setOffset(dstOffset);
  dstPosition.setLength(length);
  operation = _operation;

  if (_operand == NULL)
  {
    operand = NULL;
  }
  else
  {
    operand = new BYTE[length];
    memcpy(operand, _operand, length);
  }

  if (_mask == NULL)
  {
    mask = NULL;
  }
  else
  {
    mask = new BYTE[length];
    memcpy(mask, _mask, length);
  }
  isAffectingChksum = true;
  chksumPosition.setOffset(chksumOffset);
  chksumPosition.setLength(chksumLength);
  isPerformAction = _isPerformAction;
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other - reference to the received Action_Arithmetic, that is going to be copied.
 */
/****************************************************************************/
Action_Arithmetic::Action_Arithmetic(const Action_Arithmetic &other)
{
  this->dstPosition.setOffset(other.dstPosition.getOffset());
  this->dstPosition.setLength(other.dstPosition.getLength());
  this->operation = other.operation;

  this->operand = new BYTE[other.dstPosition.getLength()];
  memcpy(this->operand, other.operand, other.dstPosition.getLength());
  this->mask = new BYTE[other.dstPosition.getLength()];
  memcpy(this->mask, other.mask, other.dstPosition.getLength());

  this->isAffectingChksum = other.isAffectingChksum;
  this->chksumPosition.setOffset(other.chksumPosition.getOffset());
  this->chksumPosition.setLength(other.chksumPosition.getLength());
  this->isPerformAction = other.isPerformAction;
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
Action_Arithmetic::~Action_Arithmetic()
{
  if (this->operand != NULL)
    delete[] this->operand;
  if (this->mask != NULL)
    delete[] this->mask;
}

/****************************************************************************/
//! Perform this action on the received frame.
/*! Perform this action on the received frame.
 *
 * @param frm - frame structure to perform the action on.
 * @return a string that describes the result of the operation.
 */
/****************************************************************************/
string Action_Arithmetic::performAction(Frame *frm)
{
  if (dstPosition.getOffset() + dstPosition.getLength() > frm->length)
    return "Invalid offset";

  if (isAffectingChksum && chksumPosition.getOffset() + chksumPosition.getLength() > frm->length)
    return "Invalid checksum offset";

  switch (operation)
  {
    case _SUB:
      subOperation(frm);
      break;
    case _ADD:
      addOperation(frm);
      break;
    case _AND:
      andOperation(frm);
      break;
    case _XOR:
      xorOperation(frm);
      break;
    default:
      return "Invalid arithmetic operation.";
      break;
  }
  //TODO only works with length 4


  //	else if(operation.compare("IP Header checksum")==0)
  //	{
  //		unsigned int ipHeaderLen = (frm->frame[dstPosition.getOffset()-10] & 0x0f)*4;//get ip header length
  //		frm->frame[dstPosition.getOffset()]=0x00; //zero old checksum
  //		frm->frame[dstPosition.getOffset()+1]=0x00;
  //		u_int16_t newCheckSum = Checksum((u_int16_t*)(&(frm->frame[dstPosition.getOffset()-10])),ipHeaderLen); //calc checksum
  //		*((u_int16_t*)(&(frm->frame[dstPosition.getOffset()]))) = newCheckSum; //set new ip checksum
  //	}
  //	else if(operation.compare("UDP Checksum")==0)
  //	{
  //		//Calculate UDP data length
  //		unsigned int udpDataLen = 0;
  //		udpDataLen=(udpDataLen<<8) | frm->frame[dstPosition.getOffset()-2];
  //		udpDataLen=(udpDataLen<<8) | frm->frame[dstPosition.getOffset()-1];
  //		udpDataLen -= 8;
  //
  //		/*
  //                  0      7 8     15 16    23 24    31
  //                 +--------+--------+--------+--------+
  //                 |          source address           |
  //                 +--------+--------+--------+--------+
  //                 |        destination address        |
  //                 +--------+--------+--------+--------+
  //                 |  zero  |protocol|   UDP length    |
  //                 +--------+--------+--------+--------+
  //                 |     Source      |   Destination   |
  //                 |      Port       |      Port       |
  //                 +--------+--------+--------+--------+
  //                 |     Length      |    Checksum     |
  //                 +--------+--------+--------+--------+
  //                 |          data octets ...
  //                 +---------------- ...
  //		 */
  //
  //		//Create a UDP Pseudo header to calculate the checksum on
  //		BYTE *buffer = new BYTE[udpDataLen+20];
  //		memcpy(&(buffer)[0],&(frm->frame)[dstPosition.getOffset()-14],4);			//Src IP Address
  //		memcpy(&(buffer)[4],&(frm->frame)[dstPosition.getOffset()-10],4);			//Dest IP Address
  //		buffer[8]=0x0; 										//zeros
  //		buffer[9]=0x11;										//protocol (0x11 = 17 = UDP)
  //		memcpy(&(buffer)[10],&(frm->frame)[dstPosition.getOffset()-2],2);			//UDP length
  //		memcpy(&(buffer)[12],&(frm->frame)[dstPosition.getOffset()-6],6);			//UDP header
  //		buffer[18]=0x00;									//zero UDP checksum field
  //		buffer[19]=0x00;									//zero UDP checksum field
  //		memcpy(&(buffer)[20],&(frm->frame)[dstPosition.getOffset()+2],udpDataLen);	//UDP data
  //
  //		//calc checksum
  //		u_int16_t newCheckSum = Checksum((u_int16_t*)(buffer),(udpDataLen + 20));
  //		//set new UDP checksum
  //		*((u_int16_t*)(&(frm->frame[dstPosition.getOffset()]))) = newCheckSum;
  //		delete [] buffer;
  //	}
  return "DONE";
}

/****************************************************************************/
//! Prints action details.
/*! Prints action details.
 * @param ostream for cascaded output.
 */
/****************************************************************************/
void Action_Arithmetic::printAction(ostream &) const
{
  cout << "Action_Arithmetic: ";
  switch (operation)
  {
    case _SUB:
      cout << "SUB";
      break;
    case _ADD:
      cout << "ADD";
      break;
    case _AND:
      cout << "AND";
      break;
    case _XOR:
      cout << "XOR";
      break;
    default:
      cout << "Undefined Operation";
      break;
  }
  cout << ", destination offset=0x" << dstPosition.getOffset() << ", length=0x" << dstPosition.getLength()
      << ", operand=";

  Auxiliary::printBYTEarray(operand, dstPosition.getLength());
  cout << ", mask=";
  Auxiliary::printBYTEarray(mask, dstPosition.getLength());
  if (isAffectingChksum)
    cout << ", checksum field offset=0x" << chksumPosition.getOffset();
  if (!isPerformAction)
    cout << ", checksum update only(no action)";
}

/****************************************************************************/
//! Calculates checksum.
/*! Calculates checksum.
 * @param buf
 */
/****************************************************************************/
uint16_t Action_Arithmetic::Checksum(const u_int16_t* buf, unsigned int nbytes)
{
  uint32_t sum = 0;

  /* Compute Internet Checksum for "count" bytes
   *         beginning at location "addr".
   */

  while (nbytes > 1)
  {
    /*  This is the inner loop */
    sum += *buf++;
    nbytes -= 2;
  }

  /*  Add left-over byte, if any */
  if (nbytes > 0)
    sum += *(unsigned char *)buf;

  /*  Fold 32-bit sum to 16 bits */
  while (sum >> 16)
    sum = (sum & 0xffff) + (sum >> 16);

  return ~sum;
}

/****************************************************************************/
//! Performs the SUB operation.
/*! Performs the SUB operation.
 * @param frm - pointer to the frame.
 */
/****************************************************************************/
void Action_Arithmetic::subOperation(Frame *frm)
{
  BYTE *value = new BYTE[dstPosition.getLength()];

  //apply mask to field value in frame
  applyMask(frm, value);

  unsigned int i, arrToIntFrameVal = 0;
  unsigned int arrToIntOperandVal = 0;

  //convert field value and operand to integers
  for (i = 0; i < dstPosition.getLength(); i++)
  {
    arrToIntFrameVal = (arrToIntFrameVal << 8) | value[i];
    arrToIntOperandVal = (arrToIntOperandVal << 8) | operand[i];
  }

  //sub operand from frame field
  arrToIntFrameVal -= arrToIntOperandVal;

  //convert integer value to BYTEs
  unsigned int j = dstPosition.getLength() - 1;
  for (i = 0; i < dstPosition.getLength(); i++)
  {
    value[j] = (BYTE)(arrToIntFrameVal >> (i * 8));
    j--;
  }

  //update checksum if needed
  if (isAffectingChksum)
    updateChksum(frm, value);

  //store new value
  if (isPerformAction)
    memcpy(&(frm->frame)[dstPosition.getOffset()], value, dstPosition.getLength());

  delete[] value;
}

/****************************************************************************/
//! Performs the ADD operation.
/*! Performs the ADD operation.
 * @param frm - pointer to the frame.
 */
/****************************************************************************/
void Action_Arithmetic::addOperation(Frame *frm)
{
  BYTE *value = new BYTE[dstPosition.getLength()];

  //apply mask to field value in frame
  applyMask(frm, value);

  unsigned int i, arrToIntFrameVal = 0;
  unsigned int arrToIntOperandVal = 0;

  //convert field value and operand to integers
  for (i = 0; i < dstPosition.getLength(); i++)
  {
    arrToIntFrameVal = (arrToIntFrameVal << 8) | value[i];
    arrToIntOperandVal = (arrToIntOperandVal << 8) | operand[i];
  }

  //add operand to frame field
  arrToIntFrameVal += arrToIntOperandVal;

  //convert integer value to BYTEs
  unsigned int j = dstPosition.getLength() - 1;
  for (i = 0; i < dstPosition.getLength(); i++)
  {
    value[j] = (BYTE)(arrToIntFrameVal >> (i * 8));
    j--;
  }

  //update checksum if needed
  if (isAffectingChksum)
    updateChksum(frm, value);

  //store new value
  if (isPerformAction)
    memcpy(&(frm->frame)[dstPosition.getOffset()], value, dstPosition.getLength());

  delete[] value;
}

/****************************************************************************/
//! Performs the AND operation.
/*! Performs the AND operation.
 * @param frm - pointer to the frame.
 */
/****************************************************************************/
void Action_Arithmetic::andOperation(Frame *frm)
{
  BYTE *value = new BYTE[dstPosition.getLength()];

  //apply mask to field value in frame
  if (mask != NULL)
  { //apply mask and perform AND
    for (unsigned int j = 0; j < dstPosition.getLength(); ++j)
    {
      value[j] = ((frm->frame)[dstPosition.getOffset() + j]) & mask[j] & operand[j];
    }
  }
  else
  { //no mask = just perform AND
    for (unsigned int j = 0; j < dstPosition.getLength(); ++j)
    {
      value[j] = ((frm->frame)[dstPosition.getOffset() + j]) & operand[j];
    }
  }

  //update checksum if needed
  if (isAffectingChksum)
    updateChksum(frm, value);

  //store new value
  if (isPerformAction)
    memcpy(&(frm->frame)[dstPosition.getOffset()], value, dstPosition.getLength());

  delete[] value;
}

/****************************************************************************/
//! Performs the XOR operation.
/*! Performs the XOR operation.
 * @param frm - pointer to the frame.
 */
/****************************************************************************/
void Action_Arithmetic::xorOperation(Frame *frm)
{
  BYTE *value = new BYTE[dstPosition.getLength()];

  //apply mask to field value in frame
  if (mask != NULL)
  { //apply mask and perform XOR
    for (unsigned int j = 0; j < dstPosition.getLength(); ++j)
    {
      value[j] = (((frm->frame)[dstPosition.getOffset() + j]) & mask[j]) ^ operand[j];
    }
  }
  else
  { //no mask = just perform XOR
    for (unsigned int j = 0; j < dstPosition.getLength(); ++j)
    {
      value[j] = ((frm->frame)[dstPosition.getOffset() + j]) ^ operand[j];
    }
  }

  //update checksum if needed
  if (isAffectingChksum)
    updateChksum(frm, value);

  //store new value
  if (isPerformAction)
    memcpy(&(frm->frame)[dstPosition.getOffset()], value, dstPosition.getLength());

  delete[] value;
}

/****************************************************************************/
//! Updates the Checksum field according to the passed value.
/*! Updates the Checksum field according to the passed value.
 * @param frm - pointer to the frame.
 * @param value - pointer to the new value in frame that affects the checksum.
 */
/****************************************************************************/
void Action_Arithmetic::updateChksum(Frame *frm, BYTE *value)
{
  //calculate new checksum
  uint16_t newChecksum = Auxiliary::calcUpdatedChecksum(*((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))),
                                                        (u_int16_t*)(&(frm->frame)[dstPosition.getOffset()]),
                                                        (u_int16_t*)value, dstPosition.getLength());
  //update the checksum field in the frame
  *((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))) = newChecksum;
}

/****************************************************************************/
//! Applies the mask to the frame value and stores the result in 'value'.
/*! Applies the mask to the frame value and stores the result in 'value'.
 * @param frm - pointer to the frame.
 * @param value - pointer to the new value after mask operation.
 */
/****************************************************************************/
void Action_Arithmetic::applyMask(Frame *frm, BYTE *value)
{
  if (mask != NULL)
  { //apply mask
    for (unsigned int j = 0; j < dstPosition.getLength(); ++j)
    {
      value[j] = ((frm->frame)[dstPosition.getOffset() + j]) & (mask[j]);
    }
  }
  else
  { //no mask = just copy the original value
    memcpy(value, &(frm->frame)[dstPosition.getOffset()], dstPosition.getLength());
  }
}
