/****************************************************************************
 *
 * File Name: Action_Arithmetic.h
 *
 * General Description: General description of the file.
 *
 * Created By: Alex Creation Date: 01/01/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/
#ifndef ACTION_ARITHMETIC_H
#define ACTION_ARITHMETIC_H
/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file Action_Arithmetic.h
 *\brief A brief description of the file.
 *
 * Full description of the file.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action.h"

enum Operation
{
  _SUB, _ADD, _AND, _XOR
};

/**
 * \brief Action Arithmetic class.
 */
class Action_Arithmetic : public Action
{
public:
  Action_Arithmetic();
  Action_Arithmetic(unsigned int dstOffset, unsigned int length, int _operation, unsigned char *_operand,
                    unsigned char *_mask);
  Action_Arithmetic(unsigned int dstOffset, unsigned int length, int _operation, unsigned char *_operand,
                    unsigned char *_mask, unsigned int chksumOffset, unsigned int chksumLength, bool _isPerformAction);
  Action_Arithmetic(const Action_Arithmetic & other);
  virtual ~Action_Arithmetic();
  void printAction(std::ostream&) const;
  std::string performAction(Frame *frm);
  Action* clone()
  {
    return new Action_Arithmetic(*this);
  }

protected:
  Position dstPosition;
  int operation;
  BYTE *operand;
  BYTE *mask;
  bool isAffectingChksum;
  Position chksumPosition;
  bool isPerformAction;

private:
  uint16_t Checksum(const uint16_t* buf, unsigned int nbytes);
  void subOperation(Frame *frm);
  void addOperation(Frame *frm);
  void andOperation(Frame *frm);
  void xorOperation(Frame *frm);
  void updateChksum(Frame *frm, BYTE *value);
  void applyMask(Frame *frm, BYTE *value);
};

#endif /* ACTION_ARITHMETIC_H */
