/****************************************************************************
 *
 * File Name: Action_Assign.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 14/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 * 15/08/11	 Alex		 	  Added incremental checksum (rfc1141)
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Action_Assign.cpp
 *\brief Implements the Action_Assign Class.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Assign.h"

using namespace std;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign()
{
  position.setOffset(0);
  position.setLength(0);
  position.setFirstBitOffset(0);
  position.setLastBitOffset(7);
  value = new BYTE[1];
  value[0] = 0x00;
  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign(unsigned int offset, unsigned int length, BYTE *_value)
{
  position.setOffset(offset);
  position.setLength(length);
  position.setFirstBitOffset(0);
  position.setLastBitOffset(7);
  value = new BYTE[length];
  memcpy(value, _value, length);
  isAffectingChksum = false;
  //	chksumPosition.setOffset(0);
  //	chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign(unsigned int offset, unsigned int length, BYTE *_value, unsigned int firstBitOffset,
                             unsigned int lastBitOffset)
{
  position.setOffset(offset);
  position.setLength(length);
  position.setFirstBitOffset(firstBitOffset);
  position.setLastBitOffset(lastBitOffset);
  value = new BYTE[length];
  memcpy(value, _value, length);
  isAffectingChksum = false;
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign(unsigned int offset, unsigned int length, BYTE *_value, unsigned int chksumOffset,
                             unsigned int chksumLength, bool _isPerformAction)
{
  position.setOffset(offset);
  position.setLength(length);
  position.setFirstBitOffset(0);
  position.setLastBitOffset(7);
  value = new BYTE[length];
  memcpy(value, _value, length);
  isAffectingChksum = true;
  chksumPosition.setOffset(chksumOffset);
  chksumPosition.setLength(chksumLength);
  isPerformAction = _isPerformAction;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign(unsigned int offset, unsigned int length, BYTE *_value, unsigned int chksumOffset,
                             unsigned int chksumLength, bool _isPerformAction, unsigned int firstBitOffset,
                             unsigned int lastBitOffset)
{
  position.setOffset(offset);
  position.setLength(length);
  position.setFirstBitOffset(firstBitOffset);
  position.setLastBitOffset(lastBitOffset);
  value = new BYTE[length];
  memcpy(value, _value, length);
  isAffectingChksum = true;
  chksumPosition.setOffset(chksumOffset);
  chksumPosition.setLength(chksumLength);
  isPerformAction = _isPerformAction;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign(const Position _pos, BYTE *_value, unsigned int chksumOffset, unsigned int chksumLength,
                             bool _isPerformAction)
{
  position = _pos;
  value = new BYTE[_pos.getLength()];
  memcpy(value, _value, _pos.getLength());
  isAffectingChksum = true;
  chksumPosition.setOffset(chksumOffset);
  chksumPosition.setLength(chksumLength);
  isPerformAction = _isPerformAction;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Assign::Action_Assign(const Position _pos, BYTE *_value)
{
  position = _pos;
  value = new BYTE[_pos.getLength()];
  memcpy(value, _value, _pos.getLength());
  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other - reference to the received Action_Assign, that is going to be copied.
 */
/****************************************************************************/
Action_Assign::Action_Assign(const Action_Assign &other)
{
  this->position = other.position;
  this->value = new BYTE[other.position.getLength()];
  memcpy(this->value, other.value, other.position.getLength());
  this->isAffectingChksum = other.isAffectingChksum;
  this->chksumPosition.setOffset(other.chksumPosition.getOffset());
  this->chksumPosition.setLength(other.chksumPosition.getLength());
  this->isPerformAction = other.isPerformAction;
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
Action_Assign::~Action_Assign()
{
  if (this->value != NULL)
    delete[] this->value;
}

/****************************************************************************/
//! Perform this action on the received frame.
/*! Perform this action on the received frame, and update checksum if needed.
 *
 * @param frm - frame structure to perform the action on.
 * @return a string that describes the result of the operation.
 */
/****************************************************************************/
string Action_Assign::performAction(Frame *frm)
{
  if (position.getOffset() + position.getLength() > frm->length)
    return "Invalid offset";

  if (isAffectingChksum && chksumPosition.getOffset() + chksumPosition.getLength() > frm->length)
    return "Invalid checksum offset";

  if (isAffectingChksum)
  {
    uint16_t newChecksum;
    //TODO construct new value


    if (position.getFirstBitOffset() == 0 && position.getLastBitOffset() == 7)
    {
      //calculate new checksum
      newChecksum = Auxiliary::calcUpdatedChecksum(*((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))),
                                                   (u_int16_t*)(&(frm->frame)[position.getOffset()]),
                                                   (u_int16_t*)value, position.getLength());

    }
    else
    {
      BYTE *tmpValue = new BYTE[position.getLength()];

      if (position.getLength() == 1)
      {
        tmpValue[0] = (frm->frame)[position.getOffset()];
        Auxiliary::zeroBits(tmpValue, position.getFirstBitOffset(), position.getLastBitOffset(), false, true);
        tmpValue[0] |= value[0];
      }
      else
      {
        tmpValue[0] = (frm->frame)[position.getOffset()];
        Auxiliary::zeroBits(tmpValue, position.getFirstBitOffset(), 7, false, true);
        tmpValue[0] |= value[0];

        memcpy(&tmpValue[1], &value[1], position.getLength() - 1);

        tmpValue[position.getLength()] = (frm->frame)[position.getOffset() + position.getLength()];
        Auxiliary::zeroBits(&(tmpValue[position.getLength()]), 0, position.getLastBitOffset(), false, true);
        tmpValue[position.getLength()] |= value[position.getLength()];
      }

      //calculate new checksum
      newChecksum = Auxiliary::calcUpdatedChecksum(*((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))),
                                                   (u_int16_t*)(&(frm->frame)[position.getOffset()]),
                                                   (u_int16_t*)tmpValue, position.getLength());
    }

    //update the checksum field in the frame
    *((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))) = newChecksum;
  }

  if (isPerformAction)
  {

    if (position.getFirstBitOffset() == 0 && position.getLastBitOffset() == 7)
      memcpy(&(frm->frame)[position.getOffset()], value, position.getLength());
    else
    {
      if (position.getLength() == 1)
      {
        Auxiliary::zeroBits(&(frm->frame[position.getOffset()]), position.getFirstBitOffset(),
                            position.getLastBitOffset(), false, true);
        frm->frame[position.getOffset()] |= value[0];
      }
      else
      {
        Auxiliary::zeroBits(&(frm->frame[position.getOffset()]), position.getFirstBitOffset(), 7, false, true);
        //assuming irrelevant bits are zeroed
        frm->frame[position.getOffset()] |= value[0];

        memcpy(&(frm->frame)[position.getOffset() + 1], &value[1], position.getLength() - 2);

        Auxiliary::zeroBits(&(frm->frame[position.getOffset() + position.getLength() - 1]), 0,
                            position.getLastBitOffset(), false, true);
        frm->frame[position.getOffset() + position.getLength() - 1] |= value[position.getLength() - 1];
      }
    }
  }

  return "DONE";
}

/****************************************************************************/
//! Prints action details.
/*! Prints action details.
 * @param ostream for cascaded output.
 */
/****************************************************************************/
void Action_Assign::printAction(ostream&) const
{
  cout << "Action_Assign:" << position << ", value=";
  Auxiliary::printBYTEarray(value, position.getLength());
  if (isAffectingChksum)
    cout << ", checksum field offset=0x" << chksumPosition.getOffset();
  if (!isPerformAction)
    cout << ", checksum update only(no action)";
}
