/****************************************************************************
 *
 * File Name: Action_Copy.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 14/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Action_Copy.cpp
 *\brief Implements the Action_Copy Class.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Copy.h"

using namespace std;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
Action_Copy::Action_Copy()
{
  positionSrc.setOffset(0);
  positionSrc.setLength(0);
  positionDst.setOffset(0);
  positionDst.setLength(0);
  positionSrc.setFirstBitOffset(0);
  positionSrc.setLastBitOffset(7);
  positionDst.setFirstBitOffset(0);
  positionDst.setLastBitOffset(7);
  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Copy::Action_Copy(unsigned int dstOffset, unsigned int srcOffset, unsigned int length)
{
  positionSrc.setOffset(srcOffset);
  positionSrc.setLength(length);
  positionDst.setOffset(dstOffset);
  positionDst.setLength(length);
  positionSrc.setFirstBitOffset(0);
  positionSrc.setLastBitOffset(7);
  positionDst.setFirstBitOffset(0);
  positionDst.setLastBitOffset(7);
  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Copy::Action_Copy(unsigned int dstOffset, unsigned int srcOffset, unsigned int length,
                         unsigned int chksumOffset, unsigned int chksumLength, bool _isPerformAction)
{
  positionSrc.setOffset(srcOffset);
  positionSrc.setLength(length);
  positionDst.setOffset(dstOffset);
  positionDst.setLength(length);
  positionSrc.setFirstBitOffset(0);
  positionSrc.setLastBitOffset(7);
  positionDst.setFirstBitOffset(0);
  positionDst.setLastBitOffset(7);
  isAffectingChksum = true;
  chksumPosition.setOffset(chksumOffset);
  chksumPosition.setLength(chksumLength);
  isPerformAction = _isPerformAction;
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other - reference to the received Action_Copy, that is going to be copied.
 */
/****************************************************************************/
Action_Copy::Action_Copy(const Action_Copy &other)
{
  this->positionSrc = other.positionSrc;
  this->positionDst = other.positionDst;

  this->isAffectingChksum = other.isAffectingChksum;
  this->chksumPosition.setOffset(other.chksumPosition.getOffset());
  this->chksumPosition.setLength(other.chksumPosition.getLength());
  this->isPerformAction = other.isPerformAction;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Copy::Action_Copy(const Position _srcPos, const Position _dstPos, unsigned int chksumOffset,
                         unsigned int chksumLength, bool _isPerformAction)
{
  positionSrc = _srcPos;
  positionDst = _dstPos;

  isAffectingChksum = true;
  chksumPosition.setOffset(chksumOffset);
  chksumPosition.setLength(chksumLength);
  isPerformAction = _isPerformAction;
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Copy::Action_Copy(const Position _srcPos, const Position _dstPos)
{
  positionSrc = _srcPos;
  positionDst = _dstPos;

  isAffectingChksum = false;
  chksumPosition.setOffset(0);
  chksumPosition.setLength(0);
  isPerformAction = true;
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
Action_Copy::~Action_Copy()
{
}

/****************************************************************************/
//! Perform this action on the received frame.
/*! Perform this action on the received frame.
 *
 * @param frm - frame structure to perform the action on.
 * @return a string that describes the result of the operation.
 */
/****************************************************************************/
string Action_Copy::performAction(Frame *frm)
{
  if ((positionSrc.getOffset() + positionSrc.getLength() > frm->length) || (positionDst.getOffset()
      + positionDst.getLength() > frm->length))
    return "Invalid offset";

  if (isAffectingChksum && chksumPosition.getOffset() + chksumPosition.getLength() > frm->length)
    return "Invalid checksum offset";

  if (isAffectingChksum)
  {
    uint16_t newChecksum;
    //TODO construct new value


    if (positionDst.getFirstBitOffset() == 0 && positionDst.getLastBitOffset() == 7)
    {
      //calculate new checksum
      newChecksum = Auxiliary::calcUpdatedChecksum(*((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))),
                                                   (u_int16_t*)(&(frm->frame)[positionDst.getOffset()]),
                                                   (u_int16_t*)(&(frm->frame)[positionSrc.getOffset()]),
                                                   positionDst.getLength());

    }
    else
    {
      BYTE *tmpValue = new BYTE[positionDst.getLength()];
      BYTE tmpBYTE;

      if (positionDst.getLength() == 1)
      {
        tmpValue[0] = (frm->frame)[positionDst.getOffset()];
        Auxiliary::zeroBits(tmpValue, positionDst.getFirstBitOffset(), positionDst.getLastBitOffset(), false, true);
        tmpBYTE = (frm->frame)[positionSrc.getOffset()];
        Auxiliary::zeroBits(&tmpBYTE, positionSrc.getFirstBitOffset(), positionSrc.getLastBitOffset(), false, false);
        tmpValue[0] |= tmpBYTE;
      }
      else
      {
        tmpValue[0] = (frm->frame)[positionDst.getOffset()];
        Auxiliary::zeroBits(tmpValue, positionDst.getFirstBitOffset(), 7, false, true);
        tmpBYTE = (frm->frame)[positionSrc.getOffset()];
        Auxiliary::zeroBits(tmpValue, positionSrc.getFirstBitOffset(), 7, false, false);
        tmpValue[0] |= tmpBYTE;

        memmove(&tmpValue[1], &(frm->frame)[positionSrc.getOffset() + 1], positionSrc.getLength() - 2);

        tmpValue[positionDst.getLength()] = (frm->frame)[positionDst.getOffset()];
        Auxiliary::zeroBits(&tmpValue[positionDst.getLength() - 1], 0, positionDst.getLastBitOffset(), false, true);
        tmpBYTE = (frm->frame)[positionSrc.getOffset()];
        Auxiliary::zeroBits(&tmpBYTE, 0, positionDst.getLastBitOffset(), false, true);
        tmpValue[positionDst.getLength() - 1] |= tmpBYTE;
      }

      //calculate new checksum
      newChecksum = Auxiliary::calcUpdatedChecksum(*((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))),
                                                   (u_int16_t*)(&(frm->frame)[positionDst.getOffset()]),
                                                   (u_int16_t*)tmpValue, positionDst.getLength());
    }

    //update the checksum field in the frame
    *((u_int16_t*)(&(frm->frame[chksumPosition.getOffset()]))) = newChecksum;
  }

  if (isPerformAction)
  {

    if (positionSrc.getFirstBitOffset() == 0 && positionSrc.getLastBitOffset() == 7)
      memmove(&(frm->frame)[positionDst.getOffset()], &(frm->frame)[positionSrc.getOffset()], positionSrc.getLength());
    else
    {
      if (positionSrc.getLength() == 1)
      {
        BYTE newVal(frm->frame[positionSrc.getOffset()]);
        Auxiliary::zeroBits(&newVal, positionDst.getFirstBitOffset(), positionDst.getLastBitOffset(), false, false);
        Auxiliary::zeroBits(&(frm->frame[positionDst.getOffset()]), positionDst.getFirstBitOffset(),
                            positionDst.getLastBitOffset(), false, true);
        frm->frame[positionDst.getOffset()] |= newVal;
      }
      else
      {
        BYTE newVal(frm->frame[positionSrc.getOffset()]);
        Auxiliary::zeroBits(&newVal, positionDst.getFirstBitOffset(), 7, false, false);
        Auxiliary::zeroBits(&(frm->frame[positionDst.getOffset()]), positionDst.getFirstBitOffset(), 7, false, true);
        frm->frame[positionDst.getOffset()] |= newVal;

        memmove(&(frm->frame)[positionDst.getOffset() + 1], &(frm->frame)[positionSrc.getOffset() + 1],
                positionSrc.getLength() - 2);

        newVal = frm->frame[positionSrc.getOffset() + positionSrc.getLength()];
        Auxiliary::zeroBits(&newVal, 0, positionSrc.getLastBitOffset(), false, false);
        Auxiliary::zeroBits(&(frm->frame[positionDst.getOffset() + positionDst.getLength() - 1]), 0,
                            positionDst.getLastBitOffset(), false, true);
        frm->frame[positionDst.getOffset() + positionDst.getLength() - 1] |= newVal;
      }
    }

  }
  return "DONE";
}

/****************************************************************************/
//! Prints action details.
/*! Prints action details.
 * @param ostream for cascaded output.
 */
/****************************************************************************/
void Action_Copy::printAction(ostream &) const
{
  cout << "Action_Copy: source:" << positionSrc << ", destination:" << positionDst;
  if (isAffectingChksum)
    cout << ", checksum field offset=0x" << chksumPosition.getOffset();
  if (!isPerformAction)
    cout << ", checksum update only(no action)";
}
