/****************************************************************************
 *
 * File Name: Action_Cut.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 14/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Action_Cut.cpp
 *\brief Implements the Action_Cut Class.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Cut.h"

using namespace std;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
Action_Cut::Action_Cut()
{
  position.setOffset(0);
  position.setLength(0);
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Cut::Action_Cut(unsigned int offset, unsigned int length)
{
  position.setOffset(offset);
  position.setLength(length);
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other - reference to the received Action_Cut, that is going to be copied.
 */
/****************************************************************************/
Action_Cut::Action_Cut(const Action_Cut &other)
{
  this->position.setOffset(other.position.getOffset());
  this->position.setLength(other.position.getLength());
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
Action_Cut::~Action_Cut()
{
}

/****************************************************************************/
//! Perform this action on the received frame.
/*! Perform this action on the received frame.
 *
 * @param frm - frame structure to perform the action on.
 * @return a string that describes the result of the operation.
 */
/****************************************************************************/
string Action_Cut::performAction(Frame *frm)
{
  if (position.getOffset() > frm->length)
    return "Invalid offset";

  BYTE *newFramePtr = new BYTE[frm->length - position.getLength()];

  memcpy(newFramePtr, frm->frame, position.getOffset());
  memcpy(&(newFramePtr[position.getOffset()]), &(frm->frame)[position.getOffset() + position.getLength()], frm->length
      - position.getLength() - position.getOffset());
  delete[] frm->frame;

  frm->frame = newFramePtr;
  frm->length -= position.getLength();
  return "DONE";
}

/****************************************************************************/
//! Prints action details.
/*! Prints action details.
 * @param ostream for cascaded output.
 */
/****************************************************************************/
void Action_Cut::printAction(ostream &) const
{
  cout << "Action_Cut: offset=" << dec << position.getOffset() << ", length=" << dec << position.getLength();
}
