/****************************************************************************
 *
 * File Name: Action_Forward.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 16/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Action_Forward.cpp
 *\brief Implements the Action_Forward Class.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Forward.h"
#include "../Includes/Common.h"

using namespace std;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
Action_Forward::Action_Forward()
{
  PortID = 0;
  qos = 0;
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
Action_Forward::~Action_Forward()
{
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 */
/****************************************************************************/
Action_Forward::Action_Forward(unsigned int _PortID, double _qos)
{
  PortID = _PortID;
  qos = _qos;
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other - reference to the received Action_Forward, that is going to be copied.
 */
/****************************************************************************/
Action_Forward::Action_Forward(const Action_Forward &other)
{
  this->PortID = other.PortID;
  this->qos = other.qos;
}

/****************************************************************************/
//! Perform this action on the received frame.
/*! Perform this action on the received frame.
 *
 * @param frm - frame structure to perform the action on.
 * @return a string that describes the result of the operation.
 */
/****************************************************************************/
string Action_Forward::performAction(Frame *frm)
{
  if (qos == 0)
    frm->interface = this->PortID;
  else
  {
    if (((double)rand() / (double)RAND_MAX) > qos)
      frm->interface = this->PortID;
    else
      frm->interface = CE_PORT; //TODO change to drop port
  }
  return "DONE";
}

/****************************************************************************/
//! Prints action details.
/*! Prints action details.
 * @param ostream for cascaded output.
 */
/****************************************************************************/
void Action_Forward::printAction(ostream &) const
{
  cout << "Action_Forward: output port=" << PortID << ", QoS: " << qos;
}
