/****************************************************************************
 *
 * File Name: Action_Table_Impl.h
 *
 * General Description: Implements the Actions Table.
 *
 * Created By: Alex Creation Date: 09/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/
#ifndef ACTION_TABLE_IMPL_H_
#define ACTION_TABLE_IMPL_H_

//#define PATRICIA_ACTIONS

/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file Action_Table_Impl.h
 *\brief Implements the Actions Table.
 *
 * This table stores the (FWD_KEYs, Actions) pairs.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Action_Table.h"
#include "../Includes/maskedpatricia.h"

bool inline mypredicateFWD(FWD_KEY *fwdK1, FWD_KEY *fwdK2)
{
  return *fwdK1 == *fwdK2;
}

bool inline operator==(const std::vector<FWD_KEY *> &a, const std::vector<FWD_KEY *> &b)
{
  return a.size() == b.size() && std::equal(a.begin(), a.end(), b.begin(), mypredicateFWD);
}

struct fwdKeysVectHash : std::unary_function<std::vector<FWD_KEY *>, std::size_t>
{
  std::size_t operator()(std::vector<FWD_KEY *> const& key) const
  {
    std::size_t seed = 0;
    for (unsigned int i = 0; i < key.size(); ++i)
    {
      boost::hash_combine(seed, *(key.at(i)));
    }
    return seed;
  }
};

#ifdef PATRICIA_ACTIONS
typedef maskedpatricia< std::vector< std::vector<Action *> > > umultimap_act;
#else
typedef boost::unordered_multimap<std::vector<FWD_KEY *>, std::vector<Action*>, fwdKeysVectHash> umultimap_act;
#endif

/**
 * \brief Action Table Implementation class.
 */
class Action_Table_Impl : public Action_Table
{
public:
  Action_Table_Impl();
  virtual ~Action_Table_Impl();

  std::string addNewEntry(unsigned int interface, std::vector<fwdKeyMaskpair *> &fwdKeyMaskVect, std::vector<Action*> &actions);
  std::string removeEntry(unsigned int interface, std::vector<fwdKeyMaskpair *> &fwdKeyMaskVect);
  void getActions(std::vector<fwdKeyMaskpair *> &fwdKeyMaskVect, std::vector<std::vector<Action*> > &actionsVect);
  void getActions(bytes &keys, std::vector<std::vector<Action*> > &actionsVect);
  std::vector<std::vector<Action*> > *returnActions(bytes &keys);
  void printTable();
  void clearTable();

protected:
  umultimap_act actionTable; //Stores FWD_KEYs-->Actions mappings
  std::vector<std::vector<Action*> > a;
};

#endif /* ACTION_TABLE_IMPL_H_ */
