/****************************************************************************
 *
 * File Name: FWD_KEY.h
 *
 * General Description: General description of the file.
 *
 * Created By: Alex Creation Date: 05/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

#ifndef FWD_KEY_H_
#define FWD_KEY_H_

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file FWD_KEY.h
 *\brief FWD_KEY header file.
 *
 * Full description of the file.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include <string.h>
#include "../Includes/Datastructures.h"
#include "Auxiliary.h"

class FWD_KEY;
typedef std::pair<FWD_KEY, FWD_KEY> fwdKeyMaskpair;

/**
 * \brief FWD_KEY class.
 */
class FWD_KEY
{

  friend std::size_t hash_value(const FWD_KEY &key);
  friend std::ostream& operator<<(std::ostream& output, const FWD_KEY &key);
  friend std::ostream& operator<<(std::ostream& output, const FWD_KEY *key);
  friend std::ostream& operator<<(std::ostream& output, const fwdKeyMaskpair &key);
  friend std::ostream& operator<<(std::ostream& output, const fwdKeyMaskpair *key);

public:
  FWD_KEY();
  FWD_KEY(unsigned int _offset, unsigned int _length, std::string _name);
  FWD_KEY(unsigned int _offset, unsigned int _length, std::string _name, BYTE *_value);
  FWD_KEY(const FWD_KEY &other);
  FWD_KEY(const Position &_pos, std::string _name);
  FWD_KEY(const Position &_pos, std::string _name, BYTE *_value);
  virtual ~FWD_KEY();
  bool operator==(const FWD_KEY& other) const;
  unsigned int getLength() const;
  std::string getName() const;
  unsigned int getOffset() const;
  unsigned char *getValue() const;
  void setValue(unsigned char *_value);
  Position &getPosition();
  void setPosition(const Position &_pos);

  static void applyMask(std::vector<fwdKeyMaskpair *> &fwdKeyMaskVect, std::vector<FWD_KEY *> &fwdKeysWithMaskApplied);
  static void setFWD_KEYvalue(FWD_KEY &key, unsigned char *_value, bool isShiftBits);
  void adjustMaskToSize();

protected:
  Position pos;
  std::string name;
  BYTE *value;
};

#endif /* FWD_KEY_H_ */
