/****************************************************************************
 *
 * File Name: ID_KEY.cpp
 *
 * General Description: General description of the file.
 *
 * Created By: Alex

 , Creation Date: 05/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file ID_KEY.cpp
 *\brief Implements the ID_KEY Class.
 *
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
using namespace std;

#include "ID_KEY.h"

using namespace boost;

/****************************************************************************
 * functions implementations
 ****************************************************************************/

/****************************************************************************/
//! Default Constructor.
/*! Default Constructor.
 */
/****************************************************************************/
ID_KEY::ID_KEY()
{
  pos.setOffset(0);
  pos.setLength(1);
  pos.setFirstBitOffset(0);
  pos.setLastBitOffset(7);
  name = "Default ID key";
  value = new BYTE[1];
  value[0] = (0x01);
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 * @param _length - key length
 * @param _name - key name
 */
/****************************************************************************/
ID_KEY::ID_KEY(unsigned int _offset, unsigned int _length, std::string _name)
{
  pos.setOffset(_offset);
  pos.setLength(_length);
  pos.setFirstBitOffset(0);
  pos.setLastBitOffset(7);
  name = _name;
  value = new BYTE[_length];
  memset(value, 0x00, _length);
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 * @param _pos - key position
 * @param _name - key name
 */
/****************************************************************************/
ID_KEY::ID_KEY(const Position &_pos, std::string _name)
{
  pos = _pos;
  name = _name;
  value = NULL;
  BYTE _value[] = {0x00};
  setID_KEYvalue(*this, _value, false);
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 * @param _pos - key position
 * @param _name - key name
 * @param _value - key value
 */
/****************************************************************************/
ID_KEY::ID_KEY(const Position &_pos, std::string _name, BYTE *_value)
{
  pos = _pos;
  name = _name;
  value = NULL;
  setID_KEYvalue(*this, _value, false);
}

/****************************************************************************/
//! Constructor.
/*! Constructor.
 * @param _length - key length
 * @param _name - key name
 * @param _value - key value
 */
/****************************************************************************/
ID_KEY::ID_KEY(unsigned int _offset, unsigned int _length, std::string _name, BYTE *_value)
{
  pos.setOffset(_offset);
  pos.setLength(_length);
  pos.setFirstBitOffset(0);
  pos.setLastBitOffset(7);
  name = _name;
  value = new BYTE[_length];
  memmove(value, _value, _length);
}

/****************************************************************************/
//! Copy Constructor.
/*! Copy Constructor.
 * @param other reference to the received ID_KEY, that is going to be copied.
 */
/****************************************************************************/
ID_KEY::ID_KEY(const ID_KEY &other)
{
  this->pos = other.pos;
  this->name = other.name;
  this->value = NULL;
  setID_KEYvalue(*this, other.value, false);
}

/****************************************************************************/
//! Default Destructor.
/*! Default Destructor.
 */
/****************************************************************************/
ID_KEY::~ID_KEY()
{
  if (this->value != NULL)
    delete[] this->value;
}

/****************************************************************************/
//! Implementation of the hash_value function.
/*! Implementation of the hash_value function needed for boosts unordered_map.
 * @param key reference to the received ID_KEY, that is going to be hashed.
 * @return the hash value of key.
 */
/****************************************************************************/
size_t hash_value(const ID_KEY& key)
{
  size_t hashValue = 0;

  hash_combine(hashValue, key.pos.getOffset());
  hash_combine(hashValue, key.pos.getLength());
  hash_combine(hashValue, key.pos.getFirstBitOffset());
  hash_combine(hashValue, key.pos.getLastBitOffset());
  hash_combine(hashValue, key.name);
  if (key.value != NULL)
  {
    for (unsigned int i = 0; i < key.pos.getLength(); ++i)
    {
      hash_combine(hashValue, key.value[i]);
    }
  }
  return hashValue;
}

/****************************************************************************/
//! Operator '=='.
/*! Implements the '==' operator.
 * @param other reference to the received ID_KEY, that is going to be compared.
 * @return whether the Keys are equal or not.
 */
/****************************************************************************/
bool ID_KEY::operator ==(const ID_KEY& other) const
{
  return (this->pos == other.pos) && (this->name.compare(other.name) == 0) && (memcmp(this->value, other.value,
                                                                                      this->pos.getLength()
                                                                                          * sizeof(BYTE)) == 0);
}

/****************************************************************************/
//! Operator '<<'.
/*! Implements the '<<' operator.
 * @param output reference to the output of operator.
 * @param key reference to the Key that is going to be outputted.
 * @return output of operator for cascaded output.
 */
/****************************************************************************/
ostream& operator<<(ostream& output, const ID_KEY& key)
{
  output << "ID_KEY: " << key.pos << ", name=" << key.name << ", value=";
  //	if( key.pos.getLastBitOffset() > 0 )
  //		Auxiliary::printBYTEarray(key.value,key.getLength() + 1 );
  //	else
  Auxiliary::printBYTEarray(key.value, key.getLength());
  output << "]";
  return output;
}

/****************************************************************************/
//! Operator '<<'.
/*! Implements the '<<' operator.
 * @param output reference to the output of operator.
 * @param key pointer to the Key that is going to be outputted.
 * @return output of operator for cascaded output.
 */
/****************************************************************************/
ostream& operator<<(ostream& output, const ID_KEY *key)
{
  output << "ID_KEY: " << key->pos << ", name=" << key->name << ", value=";
  //	if( key->pos.getLastBitOffset() > 0 )
  //		Auxiliary::printBYTEarray(key->getValue(),key->getLength() + 1);
  //	else
  Auxiliary::printBYTEarray(key->getValue(), key->getLength());
  output << "]";
  return output;
}

/****************************************************************************/
//! Returns ID_KEY's length.
/*! Returns ID_KEY's length.
 * @return Returns ID_KEY's length.
 */
/****************************************************************************/
unsigned int ID_KEY::getLength() const
{
  return pos.getLength();
}

/****************************************************************************/
//! Returns FWD_KEY's position.
/*! Returns FWD_KEY's position.
 * @return Returns FWD_KEY's position.
 */
/****************************************************************************/
Position &ID_KEY::getPosition()
{
  return pos;
}

/****************************************************************************/
//! Sets FWD_KEY's position.
/*! Sets FWD_KEY's value.
 * @param _pos - reference to the position to set.
 */
/****************************************************************************/
void ID_KEY::setPosition(const Position &_pos)
{
  this->pos = _pos;
}

/****************************************************************************/
//! Returns ID_KEY's name.
/*! Returns ID_KEY's name.
 * @return Returns ID_KEY's name.
 */
/****************************************************************************/
std::string ID_KEY::getName() const
{
  return name;
}

/****************************************************************************/
//! Returns ID_KEY's offset.
/*! Returns ID_KEY's offset.
 * @return Returns ID_KEY's offset.
 */
/****************************************************************************/
unsigned int ID_KEY::getOffset() const
{
  return pos.getOffset();
}

/****************************************************************************/
//! Returns ID_KEY's value.
/*! Returns ID_KEY's value.
 * @return Returns ID_KEY's value.
 */
/****************************************************************************/
unsigned char *ID_KEY::getValue() const
{
  return value;
}

/****************************************************************************/
//! Sets ID_KEY's value.
/*! Sets ID_KEY's value.
 * @param _value - pointer to the value to set.
 */
/****************************************************************************/
void ID_KEY::setValue(unsigned char *_value)
{
  setID_KEYvalue(*this, _value, true);
}

/****************************************************************************/
//! Sets ID_KEY's value.
/*! Sets ID_KEY's value.
 * @param key - reference to the key to set.
 * @param _value - pointer to the value to set.
 * @param isShiftBits - if true the value in the last byte is shifted to lsb
 */
/****************************************************************************/
void ID_KEY::setID_KEYvalue(ID_KEY &key, unsigned char *_value, bool isZeroBits)
{
  //free previous value
  if (key.value != NULL)
    delete[] key.value;

  if (key.pos.getLength() > 0)
  {
    if (key.pos.getFirstBitOffset() == 0 && key.pos.getLastBitOffset() == 7)
    { //size is in bytes only
      key.value = new BYTE[key.pos.getLength()];
      memmove(key.value, _value, key.pos.getLength());
    }
    else
    { //size is in bytes and in bits
      key.value = new BYTE[key.pos.getLength()];
      memmove(key.value, _value, key.pos.getLength());
      if (isZeroBits)
      {
        if (key.pos.getLength() == 1)
        {
          Auxiliary::zeroBits(&(key.value[0]), key.pos.getFirstBitOffset(), key.pos.getLastBitOffset(), false, false);
        }
        else
        {
          if (key.pos.getFirstBitOffset() != 0)
            Auxiliary::zeroBits(&(key.value[0]), key.pos.getFirstBitOffset(), 7, false, false);

          if (key.pos.getLastBitOffset() != 7)
            Auxiliary::zeroBits(&(key.value[key.pos.getLength() - 1]), 0, key.pos.getLastBitOffset(), false, false);
        }
      }
    }
  }
  //	else
  //	{
  //		if(key.pos.getLastBitOffset() == 0)
  //		{	//size is zero
  //			key.value = NULL;
  //		}
  //		else
  //		{	//size is in bits only
  //			key.value = new BYTE[1];
  //			memmove(key.value,_value,1);
  //			if(isShiftBits)
  //				Auxiliary::zeroBits(&(key.value[0]),key.pos.getLastBitOffset(),key.pos.getFirstBitOffset(),isShiftBits,false);
  //		}
  //	}
}
