/****************************************************************************
 *
 * File Name: ID_to_FWD_Table_Impl.h
 *
 * General Description: Implements the Ident to Forward Table.
 *
 * Created By: Alex Creation Date: 09/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/
#ifndef ID_TO_FWD_TABLE_IMPL_H_
#define ID_TO_FWD_TABLE_IMPL_H_

//#define PATRICIA

/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file ID_to_FWD_Table_Impl.h
 *\brief Implements the Ident to Forward Table.
 *
 * Implements the Ident to Forward Table. This table stores the
 * (ID_KEYs,FWD_KEYs) pairs.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "ID_to_FWD_Table.h"
#include "../Includes/maskedpatricia.h"

bool inline mypredicateID(ID_KEY *idK1, ID_KEY *idK2)
{
  return *idK1 == *idK2;
}

bool inline operator==(const portIDvectPair &a, const portIDvectPair &b)
{
  return a.first == b.first && a.second.size() == b.second.size() && std::equal(a.second.begin(), a.second.end(),
                                                                                b.second.begin(), mypredicateID);
}

struct portIDvectPairHash : std::unary_function<portIDvectPair, std::size_t>
{
  std::size_t operator()(portIDvectPair const& key) const
  {
    std::size_t seed = 0;
    boost::hash_combine(seed, key.first);
    for (unsigned int i = 0; i < key.second.size(); ++i)
    {
      boost::hash_combine(seed, (*(key.second.at(i))));
    }
    return seed;
  }
};


#ifdef PATRICIA
typedef maskedpatricia<std::vector<fwdKeyMaskpair *> > umultimap_idTofwdKeys;
#else
typedef boost::unordered_multimap<portIDvectPair, std::vector<fwdKeyMaskpair *>, portIDvectPairHash> umultimap_idTofwdKeys;
#endif

/**
 * \brief ID to FWD Table Implementation class.
 */
class ID_to_FWD_Table_Impl : public ID_to_FWD_Table
{
public:
  ID_to_FWD_Table_Impl();
  virtual ~ID_to_FWD_Table_Impl();
  std::string addNewEntry(unsigned int PortID, std::vector<ID_KEY *> &idKeys,
                          std::vector<fwdKeyMaskpair *> &fwdKeysAndMasks);
  std::string removeEntry(unsigned int PortID, std::vector<ID_KEY *> &idKeys);
  void getFWD_KEYs(unsigned int PortID, std::vector<ID_KEY *> &idKeys, std::vector<fwdKeyMaskpair *> &fwdKeysVect);
  void getFWD_KEYs(bytes &keys, std::vector<fwdKeyMaskpair *> &fwdKeysVect);
  void printTable();
  void clearTable();

protected:
  umultimap_idTofwdKeys FWD_KEYs; //Stores (Port_ID,ID_KEYs)-->FWD_KEYs mappings
};

#endif /* ID_TO_FWD_TABLE_IMPL_H_ */
