/****************************************************************************
 *
 * File Name: Initial_Lookup_Table.h
 *
 * General Description: Defines the Initial Lookup Table of the LLRE.
 *
 * Created By: Alex Creation Date: 01/01/2011
 *
 * Revisions And Updates:
 *
 * Date Updated By Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/
#ifndef INITIAL_LOOKUP_TABLE_H
#define INITIAL_LOOKUP_TABLE_H
/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file Initial_Lookup_Table.h
 *\brief Defines the Initial Lookup Table of the LLRE.
 *
 * The Table stores the (Port ID, ID_KEYs) pairs, and is the first table where
 * the LLRE performs the lookup upon frame arrival. This table must store both
 * active and inactive entries.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "ID_KEY.h"

typedef struct initialLookupResult_t
{
  unsigned int entryId;
  std::vector<ID_KEY*> idKeys;
} initialLookupResult;


/**
 * \brief Initial Lookup Table class.
 */
class Initial_Lookup_Table
{
public:
  Initial_Lookup_Table(){};
  virtual ~Initial_Lookup_Table(){};
  virtual std::string addNewEntry(unsigned int PortID, bool isActive, unsigned int entryId,
                                  std::vector<ID_KEY*> &idKeys)=0;
  virtual std::string disableEntry(unsigned int entryId)=0;
  virtual std::string enableEntry(unsigned int entryId)=0;
  virtual std::string removeEntry(unsigned int PortID)=0;
  virtual void getID_KEYs(unsigned int PortID, std::vector<std::vector<ID_KEY*> > &idKeysVect)=0;
  virtual void printTable()=0;

};

#endif /* INITIAL_LOOKUP_TABLE_H */
