/****************************************************************************
 *
 * File Name: Initial_Lookup_Table_Impl.h
 *
 * General Description: Implements the Initial Lookup Table of the LLRE.
 *
 * Created By: Alex Creation Date: 07/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

#ifndef INITIAL_LOOKUP_TABLE_IMPL_H_
#define INITIAL_LOOKUP_TABLE_IMPL_H_

/****************************************************************************
 * File description
 ****************************************************************************/
/*! \file Initial_Lookup_Table_Impl.h
 *\brief Implements the Initial Lookup Table of the LLRE.
 *
 * This class implements the Initial Lookup table, by using two hash tables,
 * one for active entries, and one for inactive entries.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "Initial_Lookup_Table.h"

//typedef boost::unordered_multimap<unsigned int, initialLookupResult*> umultimap_id;
typedef boost::unordered_map<unsigned int, std::vector<initialLookupResult*> >umap_id;

/**
 * \brief Initial Lookup Table Implementation class.
 */
class Initial_Lookup_Table_Impl : public Initial_Lookup_Table
{
public:
  Initial_Lookup_Table_Impl();
  virtual ~Initial_Lookup_Table_Impl();
  std::string addNewEntry(unsigned int PortID, bool isActive, unsigned int entryId, std::vector<ID_KEY*> &idKeys);
  std::string disableEntry(unsigned int entryId);
  std::string enableEntry(unsigned int entryId);
  std::string removeEntry(unsigned int PortID);
  void getID_KEYs(unsigned int PortID, std::vector<std::vector<ID_KEY*> > &idKeysVect);
  void printTable();
  void clearTable();
  static bool resultCompare(initialLookupResult const *lhs, initialLookupResult const *rhs );
protected:
  void insertKey(umap_id &table, unsigned int PortID, initialLookupResult *result);
//  umultimap_id ID_KEYs; //Stores Port_ID-->ID_KEYs mappings
//  umultimap_id disabled_ID_KEYs; //Stores the disabled Port_ID-->ID_KEYs mappings
  umap_id ID_KEYs; //Stores Port_ID-->ID_KEYs mappings
  umap_id disabled_ID_KEYs; //Stores the disabled Port_ID-->ID_KEYs mappings
};

#endif /* INITIAL_LOOKUP_TABLE_IMPL_H_ */
