/****************************************************************************
 *
 * File Name: LLRE.h
 *
 * General Description: General description of the file.
 *
 * Created By: Alex, Creation Date: 05/04/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

#ifndef LLRE_H_
#define LLRE_H_

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file LLRE.h
 *\brief LLRE header file.
 *
 * Full description of the file.
 *
 *\author Alex
 */

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include <pthread.h>
#include <string.h>

#include "../Includes/Datastructures.h"
#include "../Includes/GFEexception.h"
#include "../Includes/concurrent_queue.h"
#include "Initial_Lookup_Table_Impl.h"
#include "ID_to_FWD_Table_Impl.h"
#include "Action_Table_Impl.h"

class Dispatcher;

/**
 * \brief Low Level Rule Enforcer (LLRE) class.
 */
class LLRE
{

  friend class LLREtest;

public:
  LLRE();
  LLRE(Dispatcher *_dispatcher);
  virtual ~LLRE();
  std::string processFrame(Frame *frm);
  std::string add_Initial_Lookup_Table_Entry(unsigned int PortID, bool isActive, unsigned int entryId, std::vector<
      ID_KEY*> &idKeys);
  std::string add_ID_to_FWD_Entry(unsigned int PortID, std::vector<ID_KEY *> &idKeys,
                                  std::vector<fwdKeyMaskpair *> &fwdKeysAndMasks);
  std::string add_Action_Table_Entry(unsigned int interface, std::vector<fwdKeyMaskpair *> &fwdKeyMaskVect, std::vector<Action*> & actions);
  std::string disableEntry(unsigned int entryId);
  std::string enableEntry(unsigned int entryId);
  void printInitialTable();
  void printIDtoFWDTable();
  void printActionTable();
  void get_Initial_Table_ID_KEYs(unsigned int PortID, std::vector<std::vector<ID_KEY*> > &idKeysVect);
  void get_ID_to_FWD_Table_FWD_KEYs(unsigned int PortID, std::vector<std::vector<ID_KEY *> > &idKeys, std::vector<
      fwdKeyMaskpair *> &fwdKeysVect);
  void get_ID_to_FWD_Table_FWD_KEYs(std::vector<bytes*> &keyBytes, std::vector<fwdKeyMaskpair *> &fwdKeysVect);
  void get_Action_Table_Actions(std::vector<fwdKeyMaskpair *> &fwdKeyMaskVect,
                                std::vector<std::vector<Action*> > & actionsVect);
  void get_Action_Table_Actions(bytes* keyByte,
                                std::vector<std::vector<Action*> > &actionsVect);
  std::vector<std::vector<Action*> > * get_Action_Table_Actions(bytes* _keyByte);
  void clearInitialTable();
  void clearIDtoFWDTable();
  void clearActionTable();
  Dispatcher *getDispatcher() const;
  void setDispatcher(Dispatcher *dispatcher);
  void pushFrameToQueue(Frame *frame);

private:
  std::string forwardFrame(Frame *frm);
  void performAllActions(std::vector<std::vector<Action*> > & actions, Frame *frm);
  void getID_KEYvaluesFromFrame(std::vector<std::vector<ID_KEY*> > &idKeys, Frame *frm);
  void getID_KEYvaluesFromFrame(std::vector<std::vector<ID_KEY*> > &idKeys, Frame *frm, std::vector<bytes *> &keysAsBytes);
  void getFWD_KEYvaluesFromFrame(std::vector<fwdKeyMaskpair*> & fwdKeys, Frame *frm);
  bytes *getFWD_KEYvaluesFromFrameAsByte(std::vector<fwdKeyMaskpair*>& fwdKeys, Frame *frm);
  void startQueueProcessingThread();
  static void *processQueue(void *_llre);

  pthread_t threadPtr; //queue processing thread id
  Initial_Lookup_Table_Impl initTable; //initial lookup table
  ID_to_FWD_Table_Impl idToFwdTable; //id to fwd table
  Action_Table_Impl actTable; //action table
  pthread_mutex_t initTable_lock; //initial table mutex
  pthread_mutex_t idToFwdTable_lock; //id to fwd table mutex
  pthread_mutex_t actTable_lock; //action table mutex
  Dispatcher *dispatcher; //dispatcher pointer
  concurrent_queue<Frame *> llreQ; //thread safe frame queue
  bytes *keyByte;
};

#endif /* LLRE_H_ */
