#include <stdio_ext.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <boost/algorithm/string.hpp>
#include "../Includes/Common.h"

#include "../GFE/GFE.h"

using namespace std;

#define FINAL
//#define HOME
//#define LAB
//#define VM

#ifdef FINAL
#define PATH ""
#endif

#ifdef VM
#define PATH "/home/b1t/workspace_GFE/XMLs/"
#define INT_1 0
#define INT_2 1
#define INT_3 2
#define INT_4 3
#endif

#ifdef HOME
#define PATH "/home/user/workspace/XMLs/"
#define INT_1 0
#define INT_2 6
#define INT_3 7
#define INT_4 8
#endif

#ifdef LAB
#define PATH "/home/uadmin/workspace/XMLs"
#define INT_1 0
#define INT_2 1
#define INT_3 2
#define INT_4 4
#endif

GFE *gfe;

/****************************************************************************/
//! Prints user menu.
/*! Prints the client menu options.
 */
/****************************************************************************/
void printMenu()
{
  if (system("clear") == -1)
    exit(1);
  cout << "╔════════════════════════════════════════╗\n";
  cout << "║               GFE menu                 ║\n";
  cout << "╠══════════════╤════════════╤════════════╣\n";
  GFE::getInstance().printXmlHandlerStatus();
  cout << "╠══════════════╧════════════╧════════════╣\n";
  cout << "║(1)Load Classifier LFB                  ║\n";
  cout << "║(2)Load Mapper LFB                      ║\n";
  cout << "║(3)Load Forwarder LFB                   ║\n";
  cout << "║(4)Load port assignment                 ║\n";
  cout << "║(5)Print Data structures                ║\n";
  cout << "║(6)Clear port assignment                ║\n";
  cout << "║(7)Disable entry                        ║\n";
  cout << "║(8)Enable entry                         ║\n";
  cout << "║(0)Quit                                 ║\n";
  cout << "╚════════════════════════════════════════╝\n";
}

void loadClassifier(string &path, stringstream &msg, string &userInput)
{
#ifndef FINAL
  msg << gfe->loadClassifier(path.append("c.xml").c_str());
#else
  cout<<"Enter classifier file name and path: ";
  getline( cin, userInput);

  msg << gfe->loadClassifier(path.append(userInput).c_str());
#endif
  printMenu();
  cout << msg.str();
}

void loadMapper(string &path, stringstream &msg, string &userInput)
{
#ifndef FINAL
  msg << gfe->loadMapper(path.append("m.xml").c_str());
#else
  cout<<"Enter mapper file name and path: ";
  getline( cin, userInput);

  msg << gfe->loadMapper(path.append(userInput).c_str());
#endif
  printMenu();
  cout << msg.str();
}

void loadForwarder(string &path, stringstream &msg, string &userInput)
{
#ifndef FINAL
  msg << gfe->loadForwarder(path.append("f.xml").c_str());
#else
  cout<<"Enter forwarder file name and path: ";
  cin >> userInput;

  msg << gfe->loadForwarder(path.append(userInput).c_str());
#endif
  printMenu();
  cout << msg.str();
}

void assignPorts(stringstream &msg, string &userInput)
{
  try
  {
#ifndef FINAL
    msg << gfe->startCapture(INT_1,-1);
#ifndef VM
    msg << gfe->startCapture(INT_2,-1);
    msg << gfe->startCapture(INT_3,-1);
    msg << gfe->startCapture(INT_4,-1);
#endif
#else
    cout<<"Available interfaces: \n";
    gfe->printInterfaces();
    cout<<"Enter port number to assign, separated by commas: ";
    getline( cin, userInput);
    boost::erase_all(userInput, " ");
    std::vector<std::string> strs;
    boost::split(strs, userInput, boost::is_any_of(","));
    for(std::vector<std::string>::iterator it = strs.begin(); it != strs.end(); ++it)
    {
      msg << gfe->startCapture(StringToNumber<unsigned int>(*it),-1);
    }
#endif

  }
  catch (NetworkDriverException e)
  {
    msg << "Network driver error: " << e.getMessage() << endl;
  }
  printMenu();
  cout << msg.str();
}

void clearPortAssignment(stringstream &msg, string &userInput)
{
  try
  {
#ifndef FINAL
    msg << gfe->stopCapture(INT_1);
#ifndef VM
    msg << gfe->stopCapture(INT_2);
    msg << gfe->stopCapture(INT_3);
    msg << gfe->stopCapture(INT_4);
#endif
#else
    cout<<"Available interfaces: \n";
    gfe->printInterfaces();
    cout<<"Enter port number to clear, separated by commas: ";
    getline( cin, userInput);

    boost::erase_all(userInput, " ");
    std::vector<std::string> strs;
    boost::split(strs, userInput, boost::is_any_of(","));
    for(std::vector<std::string>::iterator it = strs.begin(); it != strs.end(); ++it)
    {
      msg << gfe->stopCapture(StringToNumber<unsigned int>(*it));
    }
#endif
  }
  catch (NetworkDriverException e)
  {
    msg << "Network driver error: " << e.getMessage() << endl;
  }
  printMenu();
  cout << msg.str();
}

void disableEntry(stringstream &msg, string &userInput)
{
  cout << "Enter entry id number to disable: ";
  cin >> userInput;
  msg << gfe->disableEntry(StringToHexNumber<unsigned int> (userInput));
  printMenu();
  cout << msg.str();
}

void enableEntry(stringstream &msg, string &userInput)
{
  cout << "Enter entry id number to enable: ";
  cin >> userInput;
  msg << gfe->enableEntry(StringToHexNumber<unsigned int> (userInput));
  printMenu();
  cout << msg.str();
}

int menu()
{

  string path = PATH;

  int menuChoice = -1;
  gfe = &(GFE::getInstance());
  stringstream msg;
  string userInput;
  printMenu();
  do
  {
    path = PATH;
    menuChoice = Auxiliary::getch() - 48;
    __fpurge(stdin);
    printMenu();

    switch (menuChoice)
    {
      case 1:
        loadClassifier(path, msg, userInput);
        break;
      case 2:
        loadMapper(path, msg, userInput);
        break;
      case 3:
        loadForwarder(path, msg, userInput);
        break;
      case 4:
        assignPorts(msg, userInput);
        break;
      case 5:
        gfe->printInitialTable();
        gfe->printIDtoFWDTable();
        gfe->printActionTable();
        break;
      case 6:
        clearPortAssignment(msg, userInput);
        break;
      case 7:
        disableEntry(msg, userInput);
        break;
      case 8:
        enableEntry(msg, userInput);
        break;
      default:
        break;
    }
    msg.str(string());
  } while (menuChoice != 0);

  cout << gfe->StopAllActiveCapture();
  cout << "Please wait for sockets to close...\n";
  return 0;
}

int main(int argc, char **argv)
{
  srand((unsigned)time(0));
  return menu();
}

