/****************************************************************************
 *
 * File Name: Classifier_Handler.h
 *
 * General Description: Classifier_Handler header file.
 *
 * Created By: Alex Creation Date: 11/09/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

#ifndef CLASSIFIER_HANDLER_H_
#define CLASSIFIER_HANDLER_H_

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Classifier_Handler.h
 *\brief Classifier_Handler header file.
 *
 * Parses the Classifier LFB file.
 *
 *\author Alex
 */

class XML_Handler;
class ID_KEY;

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "../Includes/Datastructures.h"
#include "RapidXML/rapidxml.hpp"
//#include "RapidXML/rapidxml_iterators.hpp"
#include "RapidXML/rapidxml_print.hpp"
#include "RapidXML/rapidxml_utils.hpp"

class Classifier_Handler
{
public:
  Classifier_Handler(XML_Handler *_xmlHandler);
  virtual ~Classifier_Handler();

  std::string parse(std::vector<char> &XML);
  std::string parseFromNode(rapidxml::xml_node<>* cur_node);

private:
  void handleClassifierFrameField(std::string frame_type_id, rapidxml::xml_node<>* fields_node, Position &pos);
  bool field_node_has_all_attr(rapidxml::xml_node<>* fields_node);
  std::string processClassifierName(std::string frame_type_id, rapidxml::xml_node<>* fields_node, Position &pos);
  void handleForwardingKeyType(std::string frame_type_id, rapidxml::xml_node<>* fields_node, Position &pos,
                               std::string name);
  void handleIdentifierKeyType(std::string frame_type_id, rapidxml::xml_node<>* fields_node, Position &pos,
                               std::string name);
  void handleCheckSumKeyType(std::string frame_type_id, rapidxml::xml_node<>* fields_node, Position &pos,
                             std::string name);
  bool slowFindIdKey(ID_KEY &k);

  XML_Handler *xmlHandler;
};

#endif /* CLASSIFIER_HANDLER_H_ */
