/****************************************************************************
 *
 * File Name: Forwarder_Handler.h
 *
 * General Description: Forwarder_Handler header file.
 *
 * Created By: Alex Creation Date: 11/09/2011
 *
 * Revisions And Updates:
 *
 * Date 		 Updated By 	  Description
 * ========== ================ =========================
 *
 *
 ****************************************************************************/

#ifndef FORWARDER_HANDLER_H_
#define FORWARDER_HANDLER_H_

/****************************************************************************
 * File description
 ****************************************************************************/

/*! \file Forwarder_Handler.h
 *\brief Forwarder_Handler header file.
 *
 * Parses the Forwarder LFB file.
 *
 *\author Alex
 */

#define CLEAR_TABLES() \
	xmlHandler->llre->clearInitialTable(); \
	xmlHandler->llre->clearIDtoFWDTable(); \
	xmlHandler->llre->clearActionTable()

class XML_Handler;
class ID_KEY;
class FWD_KEY;
class Action;

/****************************************************************************
 * Include header files
 ****************************************************************************/
#include "../Includes/Datastructures.h"
#include "RapidXML/rapidxml.hpp"
//#include "RapidXML/rapidxml_iterators.hpp"
#include "RapidXML/rapidxml_print.hpp"
#include "RapidXML/rapidxml_utils.hpp"
#include "../LLRE/FWD_KEY.h"

class Forwarder_Handler
{
public:
  Forwarder_Handler(XML_Handler *_xmlHandler);
  virtual ~Forwarder_Handler();

  std::string parse(std::vector<char> &XML);
  std::string parseFromNode(rapidxml::xml_node<>* cur_node);

private:
  std::string Handle_LFB_Forwarder_fdb_entry_node(rapidxml::xml_node<>* fdb_entry_node, rapidxml::xml_node<>* cur_node);
  bool fdb_entry_has_all_attr(rapidxml::xml_node<>* fdb_entry_node);
  bool result_entry_has_all_attr(rapidxml::xml_node<>* result_node);
  void handleIncomingFrameField(rapidxml::xml_node<>* cur_node, std::vector<FWD_KEY *> &fwdKeys,
                                unsigned int interface, FWD_KEY **fwdkv, FWD_KEY **fwdkeyMask, std::vector<
                                    fwdKeyMaskpair *> &fwdKeysAndMasks);
  void handleForwarderResults(rapidxml::xml_node<>* cur_node, std::vector<fwdKeyMaskpair *> &fwdKeysAndMasks,
                              std::string frameTypeId, unsigned int interface);
  void handle_forwarder_result_frameFields(rapidxml::xml_node<> *frameFieldNode, std::vector<Action*> &actionVect,
                                           std::string frameTypeId);
  void handle_forwarder_result_arithmetics(rapidxml::xml_node<> *arithmeticsNode, std::vector<Action*> &actionVect,
                                           std::string frameTypeId);
  int findNameInFwdKeysVect(std::vector<FWD_KEY*> &fwdKeysVect, std::string &name);

  XML_Handler *xmlHandler;
};

#endif /* FORWARDER_HANDLER_H_ */
